@@ -160,6 +160,9 @@ echo -n "checking for automake >= $AUTOMAKE_REQUIRED_VERSION ... "
 if ($AUTOMAKE --version) < /dev/null > /dev/null 2>&1; then
    AUTOMAKE=$AUTOMAKE
    ACLOCAL=$ACLOCAL
+elif (automake-1.10 --version) < /dev/null > /dev/null 2>&1; then
+   AUTOMAKE=automake-1.10
+   ACLOCAL=aclocal-1.10
 elif (automake-1.9 --version) < /dev/null > /dev/null 2>&1; then
    AUTOMAKE=automake-1.9
    ACLOCAL=aclocal-1.9
@@ -240,9 +243,10 @@ test $TEST_TYPE $FILE || {
     exit 1
 }
 
-(cd operations/generated;./porter-duff.rb)
+(cd operations/generated;./svg-12-blend.rb)
+(cd operations/generated;./svg-12-porter-duff.rb)
+(cd operations/generated;./other-blend.rb)
 (cd operations/generated;./math.rb)
-(cd operations/generated;./svg-blend.rb)
 (cd operations/workshop/generated;./blend.rb)
 (cd docs/;../tools/create-reference.rb ../gegl/gegl.h api.html)
 
