@@ -2,7 +2,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -11,8 +11,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright (C) 2003, 2004, 2006 Øyvind Kolås
  */
@@ -33,6 +32,10 @@
 #define realpath(a,b) _fullpath(b,a,_MAX_PATH)
 #endif
 
+
+#define  KEY_ZOOM_FACTOR  2.0
+
+
 static gchar *blank_composition =
     "<gegl>"
         "<color value='white'/>"
@@ -148,7 +151,6 @@ create_window (Editor *editor)
   return self;
 }
 
-
 GeglNode *editor_output = NULL;
 
 static void cb_shrinkwrap (GtkAction *action);
@@ -159,6 +161,48 @@ static void cb_redraw (GtkAction *action);
 static void cb_next_file (GtkAction *action);
 static void cb_previous_file (GtkAction *action);
 
+
+static GeglNode *input_stream(GeglNode *root)
+{
+  GeglNode *iter = gegl_node_get_output_proxy (root, "output");
+  gegl_node_get_bounding_box (editor.gegl);  /* to trigger defined setting for all */
+  while (iter &&
+         gegl_node_get_producer (iter, "input", NULL)){
+    iter = gegl_node_get_producer (iter, "input", NULL);
+  }
+  return iter;
+}
+
+/* avoided to make this a really public function, since the need
+ * is a bit of a hack
+ */
+GeglProcessor *gegl_view_get_processor (GeglView *view);
+
+static gboolean play(gpointer data)
+{
+  Editor *editor=data;
+  GeglView *view=GEGL_VIEW (editor->view);
+  gdouble   progress;
+  g_object_get (gegl_view_get_processor (view), "progress", &progress, NULL);
+  if (progress >= 1.0)
+    {
+      /* modify source to trigger consumption of a new element */
+      GeglNode *source = input_stream (editor->gegl);
+      gint frame;
+      gegl_node_get (source, "frame", &frame, NULL);
+      g_warning ("(%f) frame: %i->%i", progress, frame, frame +1);
+      frame++;
+      gegl_node_set (source, "frame", frame, NULL);
+      gegl_gui_flush ();
+    }
+  else
+    {
+      g_warning ("(%f)", progress);
+    }
+
+  return TRUE;
+}
+
 static gboolean advance_slide (gpointer data)
 {
   cb_next_file (NULL);
@@ -192,6 +236,10 @@ editor_main (GeglNode    *gegl,
     {
       g_timeout_add (1000 * options->delay, advance_slide, NULL);
     }
+  if (options->play)
+    {
+      g_timeout_add (100, play, &editor);
+    }
 
   /*gtk_window_fullscreen (GTK_WINDOW (editor.window));*/
   gtk_main ();
@@ -199,7 +247,7 @@ editor_main (GeglNode    *gegl,
 }
 
 static void cb_about (GtkAction *action);
-static void cb_introspect (GtkAction *action);
+/*static void cb_introspect (GtkAction *action);*/
 static void cb_export (GtkAction *action);
 static void cb_quit_dialog (GtkAction *action);
 static void cb_composition_new (GtkAction *action);
@@ -252,11 +300,11 @@ static GtkActionEntry action_entries[] = {
    "_About", "",
    "About",
    G_CALLBACK (cb_about)},
-
+/*
   {"Introspect", NULL,
    "_Introspect", "<control>I",
    "Introspect",
-   G_CALLBACK (cb_introspect)},
+   G_CALLBACK (cb_introspect)},*/
 
   {"Export", GTK_STOCK_SAVE,
    "_Export", "<control><shift>E",
@@ -456,7 +504,7 @@ cb_composition_new (GtkAction *action)
       /* FIXME: should append to list of files, and set as current
         editor_set_gegl (gegl_parse_xml (blank_composition), "untitled.xml");
         */
-        editor_set_gegl (gegl_parse_xml (blank_composition, "/"));
+        editor_set_gegl (gegl_node_new_from_xml (blank_composition, "/"));
       }
       break;
     default:
@@ -506,7 +554,7 @@ cb_composition_load (GtkAction *action)
         {
           gchar *temp = g_strdup (filename);
           gchar *path = g_strdup (g_path_get_dirname (temp));
-          editor_set_gegl (gegl_parse_xml (xml, path));
+          editor_set_gegl (gegl_node_new_from_xml (xml, path));
           g_free (temp);
           g_free (path);
         }
@@ -575,7 +623,7 @@ cb_composition_save (GtkAction *action)
 
           realpath (full_filename, abs_filepath);
           abs_path = g_strdup (g_path_get_dirname (abs_filepath));
-          xml = gegl_to_xml (editor.gegl, abs_path); /*oxide_xml (editor->oxide, abs_path);*/
+          xml = gegl_node_to_xml (editor.gegl, abs_path); /*oxide_xml (editor->oxide, abs_path);*/
 
           g_file_set_contents (full_filename, xml, -1, NULL);
 
@@ -689,7 +737,7 @@ static void do_load (void)
       xml = g_string_free (acc, FALSE);
     }
 
-  editor_set_gegl (gegl_parse_xml (xml, path_root));
+  editor_set_gegl (gegl_node_new_from_xml (xml, path_root));
   g_free (path_root);
   g_free (xml);
 }
@@ -717,8 +765,8 @@ cb_about (GtkAction *action)
   GtkWidget *about;
   GeglNode  *gegl;
 
-  gegl = gegl_parse_xml (
-   "<gegl> <over> <invert/> <shift x='20.0' y='140.0'/> <text string=\"GEGL is a image processing and compositing framework.\n\nGUI editor Copyright © 2006, 2007 Øyvind Kolås, Kevin Cozens, Sven Neumann and Michael Schumacher\nGEGL and its editor come with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under certain conditions. The processing and compositing library GEGL is licensed under LGPLv2 and the editor itself is licensed as GPLv2.\" font='Sans' size='10.0' wrap='300' alignment='0' width='224' height='52'/> </over> <over> <shift x='20.0' y='10.0'/> <dropshadow opacity='1.0' x='10.0' y='10.0' radius='5.0'/> <text string='GEGL' font='Sans' size='100.0' wrap='-1' alignment='0'/> </over> <perlin-noise alpha='12.30' scale='0.10' zoff='-1.0' seed='20.0' n='6.0'/> </gegl>"
+  gegl = gegl_node_new_from_xml (
+   "<gegl> <over> <invert/> <shift x='20.0' y='140.0'/> <text string=\"GEGL is a image processing and compositing framework.\n\nGUI editor Copyright © 2006, 2007 Øyvind Kolås, Kevin Cozens, Sven Neumann and Michael Schumacher\nGEGL and its editor come with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under certain conditions. The processing and compositing library GEGL is licensed under LGPLv3+ and the editor itself is licensed as GPLv3+.\" font='Sans' size='10.0' wrap='300' alignment='0' width='224' height='52'/> </over> <over> <shift x='20.0' y='10.0'/> <dropshadow opacity='1.0' x='10.0' y='10.0' radius='5.0'/> <text string='GEGL' font='Sans' size='100.0' wrap='-1' alignment='0'/> </over> <perlin-noise alpha='12.30' scale='0.10' zoff='-1.0' seed='20.0' n='6.0'/> </gegl>"
   ,NULL);
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -735,6 +783,7 @@ cb_about (GtkAction *action)
   gtk_widget_show_all (window);
 }
 
+/*
 static void
 cb_introspect (GtkAction *action)
 {
@@ -767,7 +816,7 @@ cb_introspect (GtkAction *action)
                     G_CALLBACK (gtk_widget_destroy), window);
   gtk_widget_show_all (window);
 
-}
+}*/
 
 static void cb_structure_visible (GtkAction *action, gpointer userdata)
 {
@@ -812,18 +861,10 @@ static void cb_tree_visible (GtkAction *action, gpointer userdata)
 static void cb_fit (GtkAction *action)
 {
   GeglRectangle defined = gegl_node_get_bounding_box (editor.gegl);
-  gint          width;
-  gint          height;
   gint          x, y;
-  gint          i;
   gdouble       hscale;
   gdouble       vscale;
 
-
-  gtk_window_get_size (GTK_WINDOW (editor.window), &width, &height);
-  width -= editor.view->allocation.width;
-  height -= editor.view->allocation.height;
-
   hscale = (gdouble) editor.view->allocation.width / defined.width ;
   vscale = (gdouble) editor.view->allocation.height / defined.height ;
 
@@ -845,36 +886,27 @@ static void cb_fit (GtkAction *action)
                 "y",     defined.y - y,
                 "scale", hscale,
                 NULL);
-
-  width += defined.width  * hscale;
-  height += defined.height  * vscale;
-
-  for (i=0;i<40;i++){
-    gtk_main_iteration ();
-  }
 }
 
 
 static void cb_fit_on_screen (GtkAction *action)
 {
   GeglRectangle defined = gegl_node_get_bounding_box (editor.gegl);
-  /*g_warning ("shrink wrap %i,%i %ix%i", defined.x, defined.y, defined.width , defined.h);*/
+  gint ow = editor.view->allocation.width;
 
   g_object_set (editor.view, "x", defined.x, "y", defined.y, NULL);
   {
     GdkScreen *screen= gtk_window_get_screen (GTK_WINDOW (editor.window));
 
     gint    screen_width, screen_height;
-    gint i;
 
     screen_width = gdk_screen_get_width (screen);
     screen_height = gdk_screen_get_height (screen);
 
     gtk_window_resize (GTK_WINDOW (editor.window), screen_width * 0.75,
                                                    screen_height * 0.75);
-      for (i=0;i<40;i++){
-        gtk_main_iteration ();
-      }
+    while (ow == editor.view->allocation.width)
+      gtk_main_iteration();
   }
   cb_fit (action);
   cb_shrinkwrap (action);
@@ -884,7 +916,6 @@ static void cb_shrinkwrap (GtkAction *action)
 {
   GeglRectangle defined = gegl_node_get_bounding_box (editor.gegl);
   /*g_warning ("shrink wrap %i,%i %ix%i", defined.x, defined.y, defined.width , defined.h);*/
-  gint i;
 
   g_object_set (editor.view, "x", defined.x, "y", defined.y, NULL);
   {
@@ -913,17 +944,20 @@ static void cb_shrinkwrap (GtkAction *action)
 
     gtk_window_resize (GTK_WINDOW (editor.window), width, height);
   }
-
+/*
   for (i=0;i<40;i++){
     gtk_main_iteration ();
-  }
+  }*/
 }
 
 void          gegl_node_disable_cache       (GeglNode      *node);
 
 static void cb_recompute (GtkAction *action)
 {
-  gegl_node_disable_cache (GEGL_VIEW(editor.view)->node);
+  GeglNode *node;
+
+  g_object_get (GEGL_VIEW(editor.view), "node", &node, NULL);
+  gegl_node_disable_cache (node);
   gegl_gui_flush ();
 }
 
@@ -1035,60 +1069,62 @@ static void cb_zoom_50 (GtkAction *action)
 
 static void cb_zoom_in (GtkAction *action)
 {
-  gint width, height;
-  gint x,y;
+  gint    width, height;
+  gint    x, y;
   gdouble scale;
+  gdouble focus_x, focus_y;
 
-  width = editor.view->allocation.width;
+  width  = editor.view->allocation.width;
   height = editor.view->allocation.height;
 
   g_object_get (editor.view,
-                "x", &x,
-                "y", &y,
+                "x",     &x,
+                "y",     &y,
                 "scale", &scale,
                 NULL);
 
-  x += (width/2) / scale;
-  y += (height/2) / scale;
+  focus_x = (x + width  / 2) / scale;
+  focus_y = (y + height / 2) / scale;
 
-  scale *= 2.0;
+  scale *= KEY_ZOOM_FACTOR;
 
-  x -= (width/2) / scale;
-  y -= (height/2) / scale;
+  x = focus_x * scale - width  / 2;
+  y = focus_y * scale - height / 2;
 
   g_object_set (editor.view,
-                "x", x,
-                "y", y,
+                "x",     x,
+                "y",     y,
                 "scale", scale,
                 NULL);
 }
 
 static void cb_zoom_out (GtkAction *action)
 {
-  gint width, height;
-  gint x,y;
+  gint    width, height;
+  gint    x, y;
   gdouble scale;
+  gdouble focus_x, focus_y;
 
-  width = editor.view->allocation.width;
+  width  = editor.view->allocation.width;
   height = editor.view->allocation.height;
 
   g_object_get (editor.view,
-                "x", &x,
-                "y", &y,
+                "x",     &x,
+                "y",     &y,
                 "scale", &scale,
                 NULL);
 
-  x += (width/2) / scale;
-  y += (height/2) / scale;
+  focus_x = (x + width  / 2) / scale;
+  focus_y = (y + height / 2) / scale;
 
-  scale /= 2.0;
+  scale /= KEY_ZOOM_FACTOR;
 
-  x -= (width/2) / scale;
-  y -= (height/2) / scale;
+  x = focus_x * scale - width  / 2;
+  y = focus_y * scale - height / 2;
 
   g_object_set (editor.view,
-                "x", x,
-                "y", y,
+                "x",     x,
+                "y",     y,
                 "scale", scale,
                 NULL);
 
