@@ -1,3 +1,21 @@
+/* This file is part of GEGL editor -- a gtk frontend for GEGL
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
+ * Copyright (C) 2003, 2004, 2006 Øyvind Kolås
+ */
+
 #include "gegl-plugin.h"
 #include "editor.h"
 #include "export.h"
@@ -116,7 +134,7 @@ static void button_render_clicked (GtkButton *button,
   rect.height  = atoi (gtk_entry_get_text (height));
 
   path = gtk_entry_get_text (pathe);
-  node = GEGL_VIEW (editor.view)->node;
+  g_object_get (GEGL_VIEW(editor.view), "node", &node, NULL);
  
   processor = gegl_node_new_processor (node, &rect);
   
@@ -278,10 +296,10 @@ gegl_buffer_export_png (GeglBuffer    *gegl_buffer,
   strcpy (format_string, "R'G'B'A ");
 
   {
-    BablFormat  *format = gegl_buffer->format;
-    BablType   **type   = format->type;
+    Babl        *babl   = gegl_buffer->format;
+    BablType   **type   = babl->format.type;
 
-    for (i=0; i<format->components; i++)
+    for (i=0; i<babl->format.components; i++)
       if ((*type)->bits > 8)
         bit_depth = 16;
   }
@@ -328,7 +346,7 @@ gegl_buffer_export_png (GeglBuffer    *gegl_buffer,
   for (i=0; i< rect->height; i++)
     {
       GeglRectangle    line = {rect->x, rect->y + i, rect->width , 1};
-      gegl_buffer_get (gegl_buffer, &line, 1.0, babl_format (format_string), pixels);
+      gegl_buffer_get (gegl_buffer, 1.0, &line, babl_format (format_string), pixels, GEGL_AUTO_ROWSTRIDE);
       png_write_rows (png, &pixels, 1);
     }
   png_write_end (png, info);
