@@ -2,7 +2,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -11,8 +11,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright (C) 2003, 2004, 2006 Øyvind Kolås
  */
@@ -133,7 +132,7 @@ gegl_node_editor_construct (GeglNodeEditor *self)
   GeglNodeEditorClass *klass;
 
   g_return_if_fail (GEGL_IS_NODE_EDITOR (self));
-  
+
   klass = GEGL_NODE_EDITOR_GET_CLASS (self);
   g_assert (klass->construct);
   klass->construct(self);
@@ -174,7 +173,8 @@ type_editor_generic_changed (GtkWidget *entry,
   const gchar *entry_text;
   const gchar *prop_name = param_spec->name;
 
-  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
+  if (param_spec->value_type != G_TYPE_BOOLEAN)
+    entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
 
   if (param_spec->value_type == G_TYPE_INT)
     {
@@ -191,7 +191,9 @@ type_editor_generic_changed (GtkWidget *entry,
     }
   else if (param_spec->value_type == G_TYPE_BOOLEAN)
     {
-      gegl_node_set (node, prop_name, !strcmp(entry_text, "yes"), NULL);
+      gegl_node_set (node, prop_name,
+                     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (entry)),
+                     NULL);
     }
   else if (param_spec->value_type == GEGL_TYPE_COLOR)
     {
@@ -244,8 +246,8 @@ type_editor_color (GtkSizeGroup *col1,
   gtk_size_group_add_widget (col1, label);
   gtk_size_group_add_widget (col2, button);
 
-  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, FALSE, 0);
 
   g_object_set_data (G_OBJECT (button), "node", node);
 
@@ -557,7 +559,7 @@ static void scalar_expose (GtkWidget      *widget,
 
   cairo_set_font_size (cr, 12.0);
   gegl_node_get (node, param_spec->name, &value, NULL);
-  
+
   /*cairo_scale (cr, width, height);*/
   cairo_set_line_width (cr, 0.01);
   {
@@ -700,9 +702,18 @@ type_editor_generic (GtkSizeGroup *col1,
   GtkWidget *label = gtk_label_new (param_spec->name);
   GtkWidget *entry = gtk_entry_new ();
 
-  gtk_entry_set_width_chars (GTK_ENTRY (entry), 6);
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_size_group_add_widget (col1, label);
+
+  if (param_spec->value_type == G_TYPE_BOOLEAN)
+    entry = gtk_check_button_new ();
+  else
+    {
+      entry = gtk_entry_new ();
+
+      gtk_entry_set_width_chars (GTK_ENTRY (entry), 6);
+    }
+
   gtk_size_group_add_widget (col2, entry);
 
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
@@ -718,10 +729,10 @@ type_editor_generic (GtkSizeGroup *col1,
     {
       gboolean value;
       gegl_node_get (node, param_spec->name, &value, NULL);
-      if (value)
-        gtk_entry_set_text (GTK_ENTRY (entry), "yes");
-      else
-        gtk_entry_set_text (GTK_ENTRY (entry), "no");
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (entry), value);
+      g_signal_connect (G_OBJECT (entry), "toggled",
+                        G_CALLBACK (type_editor_generic_changed),
+                        (gpointer) param_spec);
     }
   else if (param_spec->value_type == G_TYPE_FLOAT)
     {
@@ -783,7 +794,7 @@ property_editor_general (GeglNodeEditor *node_editor,
 
   vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
-  
+
   properties = gegl_list_properties (gegl_node_get_operation (node), &n_properties);
   col1 = node_editor->col1;
   col2 = node_editor->col2;
@@ -796,7 +807,7 @@ property_editor_general (GeglNodeEditor *node_editor,
           strcmp (properties[i]->name, "aux"))
         {
           GtkWidget *prop_editor;
-       
+
           if (g_type_is_a (G_PARAM_SPEC_TYPE (properties[i]), GEGL_TYPE_PARAM_PATH))
             {
               prop_editor = type_editor_path (col1, col2, node, properties[i]);
@@ -816,7 +827,7 @@ property_editor_general (GeglNodeEditor *node_editor,
             }
 #endif
           else
-            { 
+            {
               prop_editor = type_editor_generic (col1, col2, node, properties[i]);
             }
           gtk_box_pack_start (GTK_BOX (vbox), prop_editor, FALSE, FALSE, 0);
@@ -906,8 +917,8 @@ static GType *gegl_type_heirs (GType  supertype,
     }
   *count = g_slist_length (subtypes);
   heirs = g_malloc (sizeof (GType) * (*count));
-   
-  for (iter = subtypes, i=0; iter; iter = g_slist_next (iter), i++) 
+
+  for (iter = subtypes, i=0; iter; iter = g_slist_next (iter), i++)
     {
       GType type = GPOINTER_TO_UINT (iter->data);
       heirs[i]=type;
