@@ -2,7 +2,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -11,8 +11,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright (C) 2003, 2004, 2006 Øyvind Kolås
  */
@@ -29,8 +28,7 @@ static GeglOptions *opts_new (void)
 
 #ifdef HAVE_GTK
   o->mode     = GEGL_RUN_MODE_EDITOR;
-#endif
-#ifndef HAVE_GTK
+#else
   o->mode     = GEGL_RUN_MODE_HELP;
 #endif
   o->xml      = NULL;
@@ -63,6 +61,9 @@ usage (char *application_name)
 "     --output    output generated image to named file\n"
 "     -o          (file is saved in PNG format)\n"
 "\n"
+"     -p          (increment frame counters of various elements when\n"
+"                  processing is done.)\n"
+"\n"
 "     -X          output the XML that was read in\n"
 "\n"
 "     --verbose   print diagnostics while running\n"
@@ -220,6 +221,10 @@ parse_args (int    argc,
             o->verbose=1;
         }
 
+        else if (match ("-p")){
+            o->play=TRUE;
+        }
+
         else if (match ("--file") ||
                  match ("-i")) {
             o->files = g_list_append (o->files, g_strdup (*curr));
