@@ -2,7 +2,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -11,8 +11,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright (C) 2003, 2004, 2006 Øyvind Kolås
  */
@@ -32,34 +31,14 @@ G_BEGIN_DECLS
 #define GEGL_IS_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  GEGL_TYPE_VIEW))
 #define GEGL_VIEW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  GEGL_TYPE_VIEW, GeglViewClass))
 
-typedef struct _GeglView      GeglView;
-typedef struct _GeglViewClass GeglViewClass;
-
+typedef struct _GeglView        GeglView;
+typedef struct _GeglViewClass   GeglViewClass;
+typedef struct _GeglViewPrivate GeglViewPrivate;
 
 struct _GeglView
 {
   GtkDrawingArea parent_instance;
 
-  /*< private >*/
-  GeglNode      *node;
-  gint           x;
-  gint           y;
-  gdouble        scale;
-  gint           screen_x;  /* coordinates of drag start */
-  gint           screen_y;
-
-  gint           orig_x;    /* coordinates of drag start */
-  gint           orig_y;
-
-  gint           start_buf_x;    /* coordinates of drag start */
-  gint           start_buf_y;
-
-  gint           prev_x;
-  gint           prev_y;
-  gdouble        prev_scale;
-
-  guint          monitor_id;
-  GeglProcessor *processor;
 };
 
 struct _GeglViewClass
@@ -67,8 +46,8 @@ struct _GeglViewClass
   GtkDrawingAreaClass parent_class;
 };
 
-GType      gegl_view_get_type  (void) G_GNUC_CONST;
-void       gegl_view_repaint   (GeglView *view);
+GType          gegl_view_get_type      (void) G_GNUC_CONST;
+void           gegl_view_repaint       (GeglView *view);
 
 G_END_DECLS
 
