@@ -2,7 +2,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -11,8 +11,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright (C) 2003, 2004, 2006 Øyvind Kolås
  */
@@ -23,12 +22,12 @@
 #include <glib-object.h>
 #include <gobject/gvaluecollector.h>
 
-#include "gegl-plugin.h"
 #include "gegl-types.h"
 #include <babl/babl.h>
-#include "gegl-node.h"
+#include "graph/gegl-node.h"
 #include "gegl-cache.h"
 #include "gegl-region.h"
+#include "../gegl-utils.h"
 
 enum
 {
@@ -40,19 +39,26 @@ enum
   PROP_NODE,
 };
 
+enum
+{
+  GEGL_CACHE_INVALIDATED,
+  GEGL_CACHE_COMPUTED,
+  GEGL_CACHE_LAST_SIGNAL
+};
 
-static void            gegl_cache_class_init     (GeglCacheClass *klass);
-static void            gegl_cache_init           (GeglCache      *self);
-static void            finalize                  (GObject       *self_object);
-static void            set_property              (GObject       *gobject,
-                                                  guint          prop_id,
-                                                  const GValue  *value,
-                                                  GParamSpec    *pspec);
-static void            get_property              (GObject       *gobject,
-                                                  guint          prop_id,
-                                                  GValue        *value,
-                                                  GParamSpec    *pspec);
-guint         gegl_cache_signals[GEGL_CACHE_LAST_SIGNAL] = {0};
+static void            gegl_cache_class_init (GeglCacheClass *klass);
+static void            gegl_cache_init (GeglCache *self);
+static void            finalize (GObject *self_object);
+static void            dispose (GObject *self_object);
+static void            set_property (GObject      *gobject,
+                                     guint         prop_id,
+                                     const GValue *value,
+                                     GParamSpec   *pspec);
+static void            get_property (GObject    *gobject,
+                                     guint       prop_id,
+                                     GValue     *value,
+                                     GParamSpec *pspec);
+guint gegl_cache_signals[GEGL_CACHE_LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (GeglCache, gegl_cache, GEGL_TYPE_BUFFER);
 
@@ -63,27 +69,28 @@ gegl_cache_constructor (GType                  type,
                         guint                  n_params,
                         GObjectConstructParam *params)
 {
-  GObject         *object;
+  GObject   *object;
   GeglCache *self;
 
   object = G_OBJECT_CLASS (parent_class)->constructor (type, n_params, params);
-  self = GEGL_CACHE (object);
+  self   = GEGL_CACHE (object);
 
   self->valid_region = gegl_region_new ();
-  self->format = GEGL_BUFFER (self)->format;
+  self->format       = GEGL_BUFFER (self)->format;
 
   return object;
 }
 
 
 static void
-gegl_cache_class_init (GeglCacheClass * klass)
+gegl_cache_class_init (GeglCacheClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->finalize     = finalize;
+  gobject_class->dispose      = dispose;
   gobject_class->set_property = set_property;
   gobject_class->get_property = get_property;
 
@@ -91,57 +98,56 @@ gegl_cache_class_init (GeglCacheClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_NODE,
                                    g_param_spec_object ("node",
-                                   "GeglNode",
-                                   "The GeglNode to cache results for",
-                                   GEGL_TYPE_NODE,
-                                   G_PARAM_WRITABLE |
-                                   G_PARAM_CONSTRUCT));
+                                                        "GeglNode",
+                                                        "The GeglNode to cache results for",
+                                                        GEGL_TYPE_NODE,
+                                                        G_PARAM_WRITABLE |
+                                                        G_PARAM_CONSTRUCT_ONLY));
 
   /* overriding pspecs for properties in parent class */
   g_object_class_install_property (gobject_class, PROP_X,
                                    g_param_spec_int ("x", "x", "local origin's offset relative to source origin",
                                                      G_MININT, G_MAXINT, -4096,
-                                                     G_PARAM_READWRITE|
+                                                     G_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property (gobject_class, PROP_Y,
                                    g_param_spec_int ("y", "y", "local origin's offset relative to source origin",
                                                      G_MININT, G_MAXINT, -4096,
-                                                     G_PARAM_READWRITE|
+                                                     G_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property (gobject_class, PROP_WIDTH,
                                    g_param_spec_int ("width", "width", "pixel width of buffer",
-                                                     -1, G_MAXINT, 10240*4,
-                                                     G_PARAM_READWRITE|
+                                                     -1, G_MAXINT, 10240 * 4,
+                                                     G_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property (gobject_class, PROP_HEIGHT,
                                    g_param_spec_int ("height", "height", "pixel height of buffer",
-                                                     -1, G_MAXINT, 10240*4,
-                                                     G_PARAM_READWRITE|
+                                                     -1, G_MAXINT, 10240 * 4,
+                                                     G_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT_ONLY));
 
 
   gegl_cache_signals[GEGL_CACHE_COMPUTED] =
-      g_signal_new ("computed", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
-      0    /* class offset*/,
-      NULL /* accumulator */,
-      NULL /* accu_data */,
-      g_cclosure_marshal_VOID__BOXED,
-      G_TYPE_NONE /* return type */,
-      1 /* n_params */,
-      GEGL_TYPE_RECTANGLE /* param_types */);
+    g_signal_new ("computed", G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
+                  0 /* class offset*/,
+                  NULL /* accumulator */,
+                  NULL /* accu_data */,
+                  g_cclosure_marshal_VOID__BOXED,
+                  G_TYPE_NONE /* return type */,
+                  1 /* n_params */,
+                  GEGL_TYPE_RECTANGLE /* param_types */);
 
   gegl_cache_signals[GEGL_CACHE_INVALIDATED] =
-      g_signal_new ("invalidated", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
-      0    /* class offset*/,
-      NULL /* accumulator */,
-      NULL /* accu_data */,
-      g_cclosure_marshal_VOID__BOXED,
-      G_TYPE_NONE /* return type */,
-      1 /* n_params */,
-      GEGL_TYPE_RECTANGLE /* param_types */);
-  
+    g_signal_new ("invalidated", G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
+                  0 /* class offset*/,
+                  NULL /* accumulator */,
+                  NULL /* accu_data */,
+                  g_cclosure_marshal_VOID__BOXED,
+                  G_TYPE_NONE /* return type */,
+                  1 /* n_params */,
+                  GEGL_TYPE_RECTANGLE /* param_types */);
 }
 
 static void
@@ -151,7 +157,29 @@ gegl_cache_init (GeglCache *self)
 
   /* thus providing a default value for GeglCache, that overrides the NULL
    * from GeglBuffer */
-  GEGL_BUFFER (self)->format = (gpointer)babl_format ("R'G'B'A u8");
+  GEGL_BUFFER (self)->format = (gpointer) babl_format ("R'G'B'A u8");
+}
+
+static void
+dispose (GObject *gobject)
+{
+  GeglCache *self = GEGL_CACHE (gobject);
+
+  while (g_idle_remove_by_data (gobject)) ;
+
+  if (self->node)
+    {
+      gint handler = g_signal_handler_find (self->node, G_SIGNAL_MATCH_DATA,
+                                            gegl_node_signals[GEGL_NODE_INVALIDATED],
+                                            0, NULL, NULL, self);
+      if (handler)
+        {
+          g_signal_handler_disconnect (self->node, handler);
+        }
+      self->node = NULL;
+    }
+
+  G_OBJECT_CLASS (gegl_cache_parent_class)->dispose (gobject);
 }
 
 static void
@@ -159,21 +187,6 @@ finalize (GObject *gobject)
 {
   GeglCache *self = GEGL_CACHE (gobject);
 
-
-  while (g_idle_remove_by_data (gobject));
-
-
-  if (self->node)
-    {
-      gint handler = g_signal_handler_find (self->node, G_SIGNAL_MATCH_DATA,
-                                                 gegl_node_signals[GEGL_NODE_INVALIDATED],
-                                                 0, NULL, NULL, self);
-      if (handler)
-        {
-          g_signal_handler_disconnect (self->node, handler);
-        }
-      g_object_unref (self->node);
-    }
   if (self->valid_region)
     gegl_region_destroy (self->valid_region);
   G_OBJECT_CLASS (gegl_cache_parent_class)->finalize (gobject);
@@ -187,10 +200,10 @@ node_invalidated (GeglNode      *source,
   GeglCache *cache = GEGL_CACHE (data);
 
   {
-      GeglRegion     *region;
-      region = gegl_region_rectangle (rect);
-      gegl_region_subtract (cache->valid_region, region);
-      gegl_region_destroy (region);
+    GeglRegion *region;
+    region = gegl_region_rectangle (rect);
+    gegl_region_subtract (cache->valid_region, region);
+    gegl_region_destroy (region);
   }
 
   g_signal_emit (cache, gegl_cache_signals[GEGL_CACHE_INVALIDATED], 0, rect, NULL);
@@ -203,12 +216,12 @@ set_property (GObject      *gobject,
               GParamSpec   *pspec)
 {
   GeglCache *self = GEGL_CACHE (gobject);
+
   switch (property_id)
     {
       case PROP_NODE:
         if (self->node)
           {
-            /* disconnecting dirt propagation */
             gulong handler;
             handler = g_signal_handler_find (self->node, G_SIGNAL_MATCH_DATA,
                                              gegl_node_signals[GEGL_NODE_INVALIDATED],
@@ -217,9 +230,11 @@ set_property (GObject      *gobject,
               {
                 g_signal_handler_disconnect (self->node, handler);
               }
-            g_object_unref (self->node);
           }
-        self->node = GEGL_NODE (g_value_dup_object (value));
+        /* just getting the node, the cache holds no reference on the node,
+         * it is the node that holds reference on the cache
+         */
+        self->node = GEGL_NODE (g_value_get_object (value));
         g_signal_connect (G_OBJECT (self->node), "invalidated",
                           G_CALLBACK (node_invalidated), self);
         break;
@@ -227,15 +242,19 @@ set_property (GObject      *gobject,
       case PROP_X:
         g_object_set_property (gobject, "GeglBuffer::x", value);
         break;
+
       case PROP_Y:
         g_object_set_property (gobject, "GeglBuffer::y", value);
         break;
+
       case PROP_WIDTH:
         g_object_set_property (gobject, "GeglBuffer::width", value);
         break;
+
       case PROP_HEIGHT:
         g_object_set_property (gobject, "GeglBuffer::height", value);
         break;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
         break;
@@ -249,35 +268,72 @@ get_property (GObject    *gobject,
               GParamSpec *pspec)
 {
   GeglCache *self = GEGL_CACHE (gobject);
+
   switch (property_id)
     {
       case PROP_NODE:
         g_value_set_object (value, self->node);
         break;
 
-      /* For the rest, upchaining to the property implementation in GeglBuffer */
+        /* For the rest, upchaining to the property implementation in GeglBuffer */
       case PROP_X:
         g_object_get_property (gobject, "GeglBuffer::x", value);
         break;
+
       case PROP_Y:
         g_object_get_property (gobject, "GeglBuffer::y", value);
         break;
+
       case PROP_WIDTH:
         g_object_get_property (gobject, "GeglBuffer::width", value);
         break;
+
       case PROP_HEIGHT:
         g_object_get_property (gobject, "GeglBuffer::height", value);
         break;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
         break;
     }
 }
 
+#if 0
+static void
+gegl_buffer_clear (GeglBuffer    *buffer,
+                   GeglRectangle *rectangle)
+{
+  gint pixels = rectangle->width * rectangle->height;
+  guchar *buf = g_malloc (pixels * 4);
+  gint i;
+
+  for (i=0;i<pixels;i++)
+    {
+      buf[i*4+0]=25;
+      buf[i*4+1]=0;
+      buf[i*4+2]=25;
+      buf[i*4+3]=40;
+    }
+  gegl_buffer_set (buffer, rectangle, babl_format ("RGBA u8"), buf);
+  g_free (buf);
+}
+#endif
+
 void
 gegl_cache_invalidate (GeglCache     *self,
                        GeglRectangle *roi)
 {
+#if 0
+  if (roi)
+    {
+      g_print ("invalidate of %i,%i %i×%i\n", roi->x, roi->y, roi->width, roi->height);
+      gegl_buffer_clear (GEGL_BUFFER (self), roi);
+    }
+  else
+    {
+      g_print ("full invalidate of a GeglCache\n");
+    }
+#endif
   if (roi)
     {
       GeglRegion *temp_region;
@@ -288,10 +344,20 @@ gegl_cache_invalidate (GeglCache     *self,
     }
   else
     {
-      GeglRectangle rect = {0,0,0,0}; /* should probably be the extent of the cache */
+      GeglRectangle rect = { 0, 0, 0, 0 }; /* should probably be the extent of the cache */
       if (self->valid_region)
         gegl_region_destroy (self->valid_region);
       self->valid_region = gegl_region_new ();
       g_signal_emit (self, gegl_cache_signals[GEGL_CACHE_INVALIDATED], 0, &rect, NULL);
     }
 }
+
+void
+gegl_cache_computed (GeglCache     *self,
+                     GeglRectangle *rect)
+{
+  g_assert (self);
+  g_assert (rect);
+  gegl_region_union_with_rect (self->valid_region, rect);
+  g_signal_emit (self, gegl_cache_signals[GEGL_CACHE_COMPUTED], 0, rect, NULL);
+}
