@@ -2,7 +2,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -11,8 +11,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright (C) 2003, 2004, 2006 Øyvind Kolås
  */
@@ -34,6 +33,8 @@ G_BEGIN_DECLS
 #define GEGL_IS_CACHE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  GEGL_TYPE_CACHE))
 #define GEGL_CACHE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  GEGL_TYPE_CACHE, GeglCacheClass))
 
+#include "gegl-buffer-private.h"
+
 struct _GeglCache
 {
   GeglBuffer    parent;
@@ -47,20 +48,11 @@ struct _GeglCacheClass
   GeglBufferClass buffer_class;
 };
 
-enum
-{
-  GEGL_CACHE_INVALIDATED,
-  GEGL_CACHE_COMPUTED,
-  GEGL_CACHE_LAST_SIGNAL
-};
-extern guint         gegl_cache_signals[GEGL_CACHE_LAST_SIGNAL];
-
 GType    gegl_cache_get_type    (void) G_GNUC_CONST;
 void     gegl_cache_invalidate  (GeglCache     *self,
                                  GeglRectangle *roi);
-gboolean gegl_cache_render      (GeglCache     *cache,
-                                 GeglRectangle *rectangle,
-                                 gdouble       *progress);
+void     gegl_cache_computed    (GeglCache     *self,
+                                 GeglRectangle *rect);
 
 G_END_DECLS
 
