@@ -3,7 +3,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,11 +11,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
+ * Copyright 2006,2007 Øyvind Kolås <pippin@gimp.org>
  */
 #include <glib.h>
 #include <glib/gprintf.h>
@@ -23,10 +21,12 @@
 #include "string.h" /* memcpy */
 #include "../gegl-types.h"
 #include "gegl-buffer.h"
+#include "gegl-buffer-private.h"
 #include "gegl-tile.h"
 
-G_DEFINE_TYPE(GeglTile, gegl_tile, G_TYPE_OBJECT)
-enum {
+G_DEFINE_TYPE (GeglTile, gegl_tile, G_TYPE_OBJECT)
+enum
+{
   PROP_0,
   PROP_X,
   PROP_Y,
@@ -42,21 +42,26 @@ get_property (GObject    *gobject,
               GValue     *value,
               GParamSpec *pspec)
 {
-  GeglTile *tile= GEGL_TILE (gobject);
-  switch(property_id)
+  GeglTile *tile = GEGL_TILE (gobject);
+
+  switch (property_id)
     {
       case PROP_X:
         g_value_set_int (value, tile->x);
         break;
+
       case PROP_Y:
         g_value_set_int (value, tile->y);
         break;
+
       case PROP_Z:
         g_value_set_int (value, tile->z);
         break;
+
       case PROP_SIZE:
         g_value_set_int (value, tile->size);
         break;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
         break;
@@ -69,18 +74,22 @@ set_property (GObject      *gobject,
               const GValue *value,
               GParamSpec   *pspec)
 {
-  GeglTile *tile= GEGL_TILE (gobject);
-  switch(property_id)
+  GeglTile *tile = GEGL_TILE (gobject);
+
+  switch (property_id)
     {
       case PROP_X:
         tile->x = g_value_get_int (value);
         return;
+
       case PROP_Y:
         tile->y = g_value_get_int (value);
         return;
+
       case PROP_Z:
         tile->z = g_value_get_int (value);
         return;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
         break;
@@ -99,8 +108,8 @@ dispose (GObject *object)
     {
       if (tile->next_shared == tile)
         { /* no clones */
-          g_free (tile->data);
-          tile->data=NULL;
+          g_slice_free1 (tile->size, tile->data);
+          tile->data = NULL;
         }
       else
         {
@@ -109,7 +118,7 @@ dispose (GObject *object)
         }
     }
 
-  (* G_OBJECT_CLASS (parent_class)->dispose) (object);
+  (*G_OBJECT_CLASS (parent_class)->dispose)(object);
 }
 
 static void
@@ -119,8 +128,8 @@ gegl_tile_class_init (GeglTileClass *class)
 
   gobject_class->set_property = set_property;
   gobject_class->get_property = get_property;
-  gobject_class->dispose = dispose;
-  parent_class = g_type_class_peek_parent (class);
+  gobject_class->dispose      = dispose;
+  parent_class                = g_type_class_peek_parent (class);
 
   g_object_class_install_property (gobject_class, PROP_X,
                                    g_param_spec_int ("x", "x", "Horizontal index",
@@ -144,21 +153,20 @@ gegl_tile_class_init (GeglTileClass *class)
 }
 
 static void
-gegl_tile_init (GeglTile* tile)
+gegl_tile_init (GeglTile *tile)
 {
-  tile->storage = NULL;
+  tile->storage    = NULL;
   tile->stored_rev = 0;
-  tile->rev = 0;
-  tile->lock = 0;
-  tile->data = NULL;
-  tile->flags = 0;
+  tile->rev        = 0;
+  tile->lock       = 0;
+  tile->data       = NULL;
+  tile->flags      = 0;
 
   tile->next_shared = tile;
   tile->prev_shared = tile;
 }
 
-
-GeglTile *
+static inline GeglTile *
 gegl_tile_new_from_data (guchar *data,
                          gint    size)
 {
@@ -179,20 +187,21 @@ gegl_tile_dup (GeglTile *src)
   tile->storage    = src->storage;
   tile->data       = src->data;
   tile->size       = src->size;
-  
-  tile->next_shared = src->next_shared;
-  src->next_shared = tile;
-  tile->prev_shared = src;
+
+  tile->next_shared              = src->next_shared;
+  src->next_shared               = tile;
+  tile->prev_shared              = src;
   tile->next_shared->prev_shared = tile;
   return tile;
 }
 
 GeglTile *
-gegl_tile_new (gint  size)
+gegl_tile_new (gint size)
 {
   GeglTile *tile;
 
-  guchar *data = g_malloc0 (size);
+  guchar   *data = g_slice_alloc (size);
+
   tile = gegl_tile_new_from_data (data, size);
 
   tile->stored_rev = 1;
@@ -204,22 +213,22 @@ gegl_tile_unclone (GeglTile *tile)
 {
   if (tile->next_shared != tile)
     {
-      gint buflen = tile->size;
+      gint    buflen = tile->size;
       /* the tile data is shared with other tiles,
        * create a local copy
        */
-      guchar *data = g_malloc (buflen);
+      guchar *data = g_slice_alloc (buflen);
       memcpy (data, tile->data, buflen);
-      tile->data = data;
+      tile->data                     = data;
       tile->prev_shared->next_shared = tile->next_shared;
       tile->next_shared->prev_shared = tile->prev_shared;
-      tile->prev_shared = tile;
-      tile->next_shared = tile;
+      tile->prev_shared              = tile;
+      tile->next_shared              = tile;
     }
 }
 
-static gint total_locks=0;
-static gint total_unlocks=0;
+static gint total_locks   = 0;
+static gint total_unlocks = 0;
 
 void
 gegl_tile_lock (GeglTile *tile)
@@ -246,36 +255,37 @@ gegl_tile_void_pyramid (GeglTile *tile)
    * similar to how it was done in horizon, this will only work with 4->1 px
    * averageing.
    */
-  gint x,y,z;
+  gint x, y, z;
+
   x = tile->storage_x;
   y = tile->storage_y;
   z = 0;/*tile->storage_z;*/
 
-  for (z=1; z<10;z++)
+  for (z = 1; z < 10; z++)
     {
 #if 0
-      gint ver = (y%2);
-      gint hor = (x%2);
+      gint ver = (y % 2);
+      gint hor = (x % 2);
 #endif
-      x/=2;
-      y/=2;
+      x /= 2;
+      y /= 2;
 
-      gegl_tile_store_message (GEGL_TILE_STORE (tile->storage),
+      gegl_provider_message (GEGL_PROVIDER (tile->storage),
                                GEGL_TILE_VOID,
-                               x,y,z,NULL);
+                               x, y, z, NULL);
 #if 0
-  /* FIXME: reenable this code */
+      /* FIXME: reenable this code */
       if (!ver)
         {
           if (!hor)
             {
-              gegl_tile_store_message (GEGL_TILE_STORE (tile->storage),
+              gegl_provider_message (GEGL_PROVIDER (tile->storage),
                                        GEGL_TILE_VOID_TL,
                                        x, y, z, NULL);
             }
           else
             {
-              gegl_tile_store_message (GEGL_TILE_STORE (tile->storage),
+              gegl_provider_message (GEGL_PROVIDER (tile->storage),
                                        GEGL_TILE_VOID_TR,
                                        x, y, z, NULL);
             }
@@ -284,13 +294,13 @@ gegl_tile_void_pyramid (GeglTile *tile)
         {
           if (!hor)
             {
-              gegl_tile_store_message (GEGL_TILE_STORE (tile->storage),
+              gegl_provider_message (GEGL_PROVIDER (tile->storage),
                                        GEGL_TILE_VOID_BL,
                                        x, y, z, NULL);
             }
           else
             {
-              gegl_tile_store_message (GEGL_TILE_STORE (tile->storage),
+              gegl_provider_message (GEGL_PROVIDER (tile->storage),
                                        GEGL_TILE_VOID_BR,
                                        x, y, z, NULL);
             }
@@ -324,7 +334,7 @@ gegl_tile_is_stored (GeglTile *tile)
 }
 
 void
-gegl_tile_void (GeglTile       *tile)
+gegl_tile_void (GeglTile *tile)
 {
   tile->stored_rev = tile->rev;
   /* FIXME: make sure the tile is evicted from any storage/buffer caches
@@ -337,13 +347,13 @@ gegl_tile_cpy (GeglTile *src,
                GeglTile *dst)
 {
   gegl_tile_lock (dst);
-  
-  g_free (dst->data);
+
+  g_slice_free1 (dst->size, dst->data);
   dst->data = NULL;
-  
-  dst->next_shared = src->next_shared;
-  src->next_shared = dst;
-  dst->prev_shared = src;
+
+  dst->next_shared              = src->next_shared;
+  src->next_shared              = dst;
+  dst->prev_shared              = src;
   dst->next_shared->prev_shared = dst;
 
   dst->data = src->data;
@@ -356,16 +366,16 @@ gegl_tile_swp (GeglTile *a,
                GeglTile *b)
 {
   guchar *tmp;
- 
+
   gegl_tile_unclone (a);
   gegl_tile_unclone (b);
 
 /*  gegl_buffer_add_dirty (a->buffer, a->x, a->y);
-  gegl_buffer_add_dirty (b->buffer, b->x, b->y);*/
+   gegl_buffer_add_dirty (b->buffer, b->x, b->y);*/
 
   g_assert (a->size == b->size);
 
-  tmp = a->data;
+  tmp     = a->data;
   a->data = b->data;
   b->data = tmp;
 }
@@ -376,14 +386,14 @@ gegl_tile_get_data (GeglTile *tile)
   return tile->data;
 }
 
-gboolean gegl_tile_store (GeglTile   *tile)
+gboolean gegl_tile_store (GeglTile *tile)
 {
   if (tile->storage == NULL)
     return FALSE;
-  return gegl_tile_store_message (GEGL_TILE_STORE (tile->storage),
-                                  GEGL_TILE_SET, tile->storage_x, 
-                                                 tile->storage_y,
-                                                 tile->storage_z, tile);
+  return gegl_provider_message (GEGL_PROVIDER (tile->storage),
+                                  GEGL_TILE_SET, tile->storage_x,
+                                  tile->storage_y,
+                                  tile->storage_z, tile);
 }
 
 
@@ -394,17 +404,18 @@ gint
 gegl_tile_indice (gint coordinate,
                   gint stride)
 {
-  if (coordinate>=0)
-    return coordinate/stride;
-  return (((coordinate+1)/stride)-1);
+  if (coordinate >= 0)
+    return coordinate / stride;
+  return (((coordinate + 1) / stride) - 1);
 }
 
 /* computes the positive integer remainder (also for negative dividends)
  */
-#define REMAINDER(dividend, divisor)                       \
-   ((((dividend) < 0) ?                                    \
-   ((divisor) - ((-(dividend)) % (divisor))) % (divisor) : \
-   ((dividend) % (divisor))))
+#define REMAINDER(dividend, divisor) \
+                   (((dividend) < 0) ? \
+                    (divisor) - 1 - ((-((dividend) + 1)) % (divisor)) : \
+                    (dividend) % (divisor))
+
 
 /* compute the offset into the containing tile a coordinate has,
  * the stride is the width/height of tiles along the axis of coordinate
@@ -413,5 +424,5 @@ gint
 gegl_tile_offset (gint coordinate,
                   gint stride)
 {
-  return REMAINDER (coordinate,stride);
+  return REMAINDER (coordinate, stride);
 }
