@@ -3,7 +3,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,11 +11,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
+ * Copyright 2006,2007 Øyvind Kolås <pippin@gimp.org>
  */
 #ifndef GEGL_TILE_H
 #define GEGL_TILE_H
@@ -77,8 +75,6 @@ struct _GeglTileClass
 GType  gegl_tile_get_type (void) G_GNUC_CONST;
 
 GeglTile   * gegl_tile_new             (gint         size);
-GeglTile   * gegl_tile_new_from_data   (guchar      *data,
-                                        gint         size);
 guchar     * gegl_tile_get_data        (GeglTile    *tile);
 void       * gegl_tile_get_format      (GeglTile    *tile);
 void         gegl_tile_lock            (GeglTile    *tile);
