@@ -12,50 +12,57 @@
 #include <gegl-plugin.h>
 
 #ifdef GEGL_CHANT_SOURCE
-  #include <gegl/gegl-operation-source.h>
-  #include <gegl/gegl-extension-handler.h>
+  #include <gegl/operation/gegl-operation-source.h>
+  #include <gegl/operation/gegl-extension-handler.h>
   #define GEGL_CHANT_PARENT_TypeName      GeglOperationSource
   #define GEGL_CHANT_PARENT_TypeNameClass GeglOperationSourceClass
   #define GEGL_CHANT_PARENT_TYPE          GEGL_TYPE_OPERATION_SOURCE
   #define GEGL_CHANT_PARENT_CLASS         GEGL_OPERATION_SOURCE_CLASS
 #endif
 #ifdef GEGL_CHANT_SINK
-  #include <gegl/gegl-operation-sink.h>
+  #include <gegl/operation/gegl-operation-sink.h>
   #define GEGL_CHANT_PARENT_TypeName      GeglOperationSink
   #define GEGL_CHANT_PARENT_TypeNameClass GeglOperationSinkClass
   #define GEGL_CHANT_PARENT_TYPE          GEGL_TYPE_OPERATION_SINK
   #define GEGL_CHANT_PARENT_CLASS         GEGL_OPERATION_SINK_CLASS
 #endif
 #ifdef GEGL_CHANT_FILTER
-  #include <gegl/gegl-operation-filter.h>
+  #include <gegl/operation/gegl-operation-filter.h>
   #define GEGL_CHANT_PARENT_TypeName      GeglOperationFilter
   #define GEGL_CHANT_PARENT_TypeNameClass GeglOperationFilterClass
   #define GEGL_CHANT_PARENT_TYPE          GEGL_TYPE_OPERATION_FILTER
   #define GEGL_CHANT_PARENT_CLASS         GEGL_OPERATION_FILTER_CLASS
 #endif
 #ifdef GEGL_CHANT_POINT_FILTER
-  #include <gegl/gegl-operation-point-filter.h>
+  #include <gegl/operation/gegl-operation-point-filter.h>
   #define GEGL_CHANT_PARENT_TypeName      GeglOperationPointFilter
   #define GEGL_CHANT_PARENT_TypeNameClass GeglOperationPointFilterClass
   #define GEGL_CHANT_PARENT_TYPE          GEGL_TYPE_OPERATION_POINT_FILTER
   #define GEGL_CHANT_PARENT_CLASS         GEGL_OPERATION_POINT_FILTER_CLASS
 #endif
+#ifdef GEGL_CHANT_AREA_FILTER
+  #include <gegl/operation/gegl-operation-area-filter.h>
+  #define GEGL_CHANT_PARENT_TypeName      GeglOperationAreaFilter
+  #define GEGL_CHANT_PARENT_TypeNameClass GeglOperationAreaFilterClass
+  #define GEGL_CHANT_PARENT_TYPE          GEGL_TYPE_OPERATION_AREA_FILTER
+  #define GEGL_CHANT_PARENT_CLASS         GEGL_OPERATION_AREA_FILTER_CLASS
+#endif
 #ifdef GEGL_CHANT_COMPOSER
-  #include <gegl/gegl-operation-composer.h>
+  #include <gegl/operation/gegl-operation-composer.h>
   #define GEGL_CHANT_PARENT_TypeName      GeglOperationComposer
   #define GEGL_CHANT_PARENT_TypeNameClass GeglOperationComposerClass
   #define GEGL_CHANT_PARENT_TYPE          GEGL_TYPE_OPERATION_COMPOSER
   #define GEGL_CHANT_PARENT_CLASS         GEGL_OPERATION_COMPOSER_CLASS
 #endif
 #ifdef GEGL_CHANT_POINT_COMPOSER
-  #include <gegl/gegl-operation-point-composer.h>
+  #include <gegl/operation/gegl-operation-point-composer.h>
   #define GEGL_CHANT_PARENT_TypeName      GeglOperationPointComposer
   #define GEGL_CHANT_PARENT_TypeNameClass GeglOperationPointComposerClass
   #define GEGL_CHANT_PARENT_TYPE          GEGL_TYPE_OPERATION_POINT_COMPOSER
   #define GEGL_CHANT_PARENT_CLASS         GEGL_OPERATION_POINT_COMPOSER_CLASS
 #endif
 #ifdef GEGL_CHANT_META
-  #include <gegl/gegl-operation-meta.h>
+  #include <gegl/operation/gegl-operation-meta.h>
   #define GEGL_CHANT_PARENT_TypeName      GeglOperationMeta
   #define GEGL_CHANT_PARENT_TypeNameClass GeglOperationMetaClass
   #define GEGL_CHANT_PARENT_TYPE          GEGL_TYPE_OPERATION_META
@@ -78,6 +85,7 @@ struct Generated
 #define gegl_chant_object(name, blurb)                 GObject   *name;
 #define gegl_chant_pointer(name, blurb)                gpointer   name;
 #define gegl_chant_color(name, def, blurb)             GeglColor *name;
+#define gegl_chant_curve(name, blurb)                  GeglCurve *name;
 
 #include GEGL_CHANT_SELF
 
@@ -94,6 +102,7 @@ struct Generated
 #undef gegl_chant_object
 #undef gegl_chant_pointer
 #undef gegl_chant_color
+#undef gegl_chant_curve
 
 /****************************************************************************/
 
@@ -231,6 +240,7 @@ enum
 #define gegl_chant_object(name, blurb)                 PROP_##name,
 #define gegl_chant_pointer(name, blurb)                PROP_##name,
 #define gegl_chant_color(name, def, blurb)             PROP_##name,
+#define gegl_chant_curve(name, blurb)                  PROP_##name,
 
 #include GEGL_CHANT_SELF
 
@@ -243,6 +253,7 @@ enum
 #undef gegl_chant_object
 #undef gegl_chant_pointer
 #undef gegl_chant_color
+#undef gegl_chant_curve
   PROP_LAST
 };
 
@@ -274,6 +285,8 @@ get_property (GObject      *gobject,
     case PROP_##name: g_value_set_pointer (value, self->name);break;
 #define gegl_chant_color(name, def, blurb)\
     case PROP_##name: g_value_set_object (value, self->name);break;
+#define gegl_chant_curve(name, blurb)\
+    case PROP_##name: g_value_set_object (value, self->name);break;
 
 #include GEGL_CHANT_SELF
 
@@ -286,6 +299,7 @@ get_property (GObject      *gobject,
 #undef gegl_chant_object
 #undef gegl_chant_pointer
 #undef gegl_chant_color
+#undef gegl_chant_curve
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
       break;
@@ -347,7 +361,13 @@ set_property (GObject      *gobject,
     case PROP_##name:\
       if (self->name != NULL && G_IS_OBJECT (self->name))\
          g_object_unref (self->name);\
-      self->name = g_value_get_object (value);\
+      self->name = g_value_dup_object (value);\
+      break;
+#define gegl_chant_curve(name, blurb)\
+    case PROP_##name:\
+      if (self->name != NULL && G_IS_OBJECT (self->name))\
+         g_object_unref (self->name);\
+      self->name = g_value_dup_object (value);\
       break;
 
 #include GEGL_CHANT_SELF
@@ -361,6 +381,7 @@ set_property (GObject      *gobject,
 #undef gegl_chant_object
 #undef gegl_chant_pointer
 #undef gegl_chant_color
+#undef gegl_chant_curve
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
@@ -399,6 +420,15 @@ gegl_chant_init (GeglChantOperation *self)
 static void init (GeglChantOperation *self);
 #endif
 
+#ifdef GEGL_CHANT_PREPARE
+static void prepare (GeglOperation *self,
+                     gpointer       context_id);
+#endif
+
+#ifdef GEGL_CHANT_AREA_FILTER
+static void tickle (GeglOperation *self);
+#endif
+
 static void gegl_chant_destroy_notify (gpointer data)
 {
   GeglChantOperation *self = GEGL_CHANT_OPERATION (data);
@@ -437,6 +467,12 @@ static void gegl_chant_destroy_notify (gpointer data)
       g_object_unref (self->name);\
       self->name = NULL;\
     }
+#define gegl_chant_curve(name, blurb)\
+  if (self->name)\
+    {\
+      g_object_unref (self->name);\
+      self->name = NULL;\
+    }
 
 #include GEGL_CHANT_SELF
 
@@ -449,6 +485,7 @@ static void gegl_chant_destroy_notify (gpointer data)
 #undef gegl_chant_object
 #undef gegl_chant_pointer
 #undef gegl_chant_color
+#undef gegl_chant_curve
   self = NULL; /* shut up gcc warnings if there were no properties to clean up*/
 }
 
@@ -486,12 +523,25 @@ gegl_chant_class_init (ChantClass * klass)
   GeglOperationClass         *operation_class;
 
 #ifndef GEGL_CHANT_META
+#ifndef GEGL_CHANT_AREA_FILTER
   GEGL_CHANT_PARENT_TypeNameClass *parent_class = GEGL_CHANT_PARENT_CLASS (klass);
+#endif
+#ifdef GEGL_CHANT_AREA_FILTER
+  /* not really the parent class, but what we're after */
+  GeglOperationFilterClass *parent_class = GEGL_OPERATION_FILTER_CLASS (klass);
+#endif
   parent_class->process = process;
 #endif
-
   operation_class = GEGL_OPERATION_CLASS (klass);
 
+#ifdef GEGL_CHANT_PREPARE
+  operation_class->prepare = prepare;
+#endif
+
+#ifdef GEGL_CHANT_AREA_FILTER
+  operation_class->tickle = tickle;
+#endif
+
   object_class->set_property = set_property;
   object_class->get_property = get_property;
 
@@ -511,9 +561,8 @@ gegl_chant_class_init (ChantClass * klass)
   M_GEGL_CHANT_SET_NAME (GEGL_CHANT_NAME);
 
 #ifdef GEGL_CHANT_DESCRIPTION
-  gegl_operation_class_set_description (operation_class, GEGL_CHANT_DESCRIPTION);
+  operation_class->description = GEGL_CHANT_DESCRIPTION;
 #endif
-
 #ifdef GEGL_CHANT_CATEGORIES
   operation_class->categories = GEGL_CHANT_CATEGORIES;
 #else
@@ -592,6 +641,14 @@ gegl_chant_class_init (ChantClass * klass)
                                                           G_PARAM_READWRITE |\
                                                           G_PARAM_CONSTRUCT |\
                                                           GEGL_PAD_INPUT)));
+#define gegl_chant_curve(name, blurb)  \
+  g_object_class_install_property (object_class, PROP_##name,\
+                                   gegl_param_spec_curve (#name, #name, blurb,\
+							  gegl_curve_default_curve(),\
+                                                          (GParamFlags) (\
+                                                          G_PARAM_READWRITE |\
+                                                          G_PARAM_CONSTRUCT |\
+                                                          GEGL_PAD_INPUT)));
 #include GEGL_CHANT_SELF
 
 #undef gegl_chant_int
@@ -603,6 +660,7 @@ gegl_chant_class_init (ChantClass * klass)
 #undef gegl_chant_object
 #undef gegl_chant_pointer
 #undef gegl_chant_color
+#undef gegl_chant_curve
 }
 
 
