@@ -3,7 +3,7 @@
  * GEGL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * GEGL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,11 +11,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright 2003 Calvin Williamson
+ * Copyright 2003-2007 Calvin Williamson, Øyvind Kolås
  */
 
 #include "config.h"
@@ -29,6 +27,8 @@
 #include "module/geglmoduledb.h"
 #include <stdlib.h>
 #include <unistd.h>
+#include <glib/gstdio.h>
+#include "operation/gegl-extension-handler.h"
 
 static gboolean  gegl_post_parse_hook (GOptionContext *context,
                                        GOptionGroup   *group,
@@ -38,7 +38,7 @@ static gboolean  gegl_post_parse_hook (GOptionContext *context,
 
 static gboolean gegl_initialized = FALSE;
 
-static glong global_time = 0;
+static glong    global_time = 0;
 
 
 /**
@@ -57,11 +57,13 @@ static glong global_time = 0;
  * by gegl_get_option_group(), you don't have to call gegl_init().
  **/
 void
-gegl_init (gint *argc,
+gegl_init (gint    *argc,
            gchar ***argv)
 {
   if (gegl_initialized)
     return;
+  if (!g_thread_supported())
+    g_thread_init (NULL);
 
   /*  If any command-line actions are ever added to GEGL, then the
    *  commented out code below should be used.  Until then, we simply
@@ -71,14 +73,14 @@ gegl_init (gint *argc,
 
 #if 0
   GOptionContext *context;
-  GError *error = NULL;
+  GError         *error = NULL;
 
   context = g_option_context_new (NULL);
   g_option_context_set_ignore_unknown_options (context, TRUE);
   g_option_context_set_help_enabled (context, FALSE);
   g_option_context_set_main_group (context, gegl_get_option_group ());
 
-  if (! g_option_context_parse (context, argc, argv, &error))
+  if (!g_option_context_parse (context, argc, argv, &error))
     {
       g_warning ("%s", error->message);
       g_error_free (error);
@@ -117,13 +119,15 @@ void
 gegl_exit (void)
 {
   glong timing = gegl_ticks ();
+
+  gegl_extension_handler_cleanup ();
   gegl_buffer_allocators_free ();
   babl_destroy ();
   timing = gegl_ticks () - timing;
   gegl_instrument ("gegl", "gegl_exit", timing);
 
   /* used when tracking buffer and tile leaks */
-  if(g_getenv ("GEGL_DEBUG_BUFS")!=NULL)
+  if (g_getenv ("GEGL_DEBUG_BUFS") != NULL)
     {
       gegl_buffer_stats ();
       gegl_tile_mem_stats ();
@@ -131,21 +135,44 @@ gegl_exit (void)
   global_time = gegl_ticks () - global_time;
   gegl_instrument ("gegl", "gegl", global_time);
 
-  if (g_getenv ("GEGL_DEBUG_TIME")!=NULL)
+  if (g_getenv ("GEGL_DEBUG_TIME") != NULL)
     {
       g_print ("\n%s", gegl_instrument_utf8 ());
     }
 
-  if (gegl_buffer_leaks())
+  if (gegl_buffer_leaks ())
     g_print ("  buffer-leaks: %i", gegl_buffer_leaks ());
 
   if (g_getenv ("GEGL_SWAP"))
     {
-      const gchar *swapdir = g_getenv ("GEGL_SWAP");
-      guint pid = getpid ();
-      gchar buf[4096];
-      g_snprintf (buf, 4096, "rm %s/GEGL-%i-*.swap", swapdir, pid);
-      system (buf);
+      /* remove all files matching <$GEGL_SWAP>/GEGL-<pid>-*.swap */
+
+      const gchar  *swapdir = g_getenv ("GEGL_SWAP");
+      guint         pid     = getpid ();
+      GDir         *dir     = g_dir_open (swapdir, 0, NULL);
+
+      gchar        *glob    = g_strdup_printf ("GEGL-%i-*.swap", pid);
+      GPatternSpec *pattern = g_pattern_spec_new (glob);
+      g_free (glob);
+
+      if (dir != NULL)
+        {
+          const gchar *name;
+
+          while ((name = g_dir_read_name (dir)) != NULL)
+            {
+              if (g_pattern_match_string (pattern, name))
+                {
+                  gchar *fname = g_strdup_printf ("%s/%s", swapdir, name);
+                  g_unlink (fname);
+                  g_free (fname);
+                }
+            }
+
+          g_dir_close (dir);
+        }
+
+      g_pattern_spec_free (pattern);
     }
 
   g_print ("\n");
@@ -158,7 +185,7 @@ gegl_post_parse_hook (GOptionContext *context,
                       GError        **error)
 {
   static GeglModuleDB *module_db = NULL;
-  glong time;
+  glong                time;
 
   if (gegl_initialized)
     return TRUE;
@@ -178,7 +205,8 @@ gegl_post_parse_hook (GOptionContext *context,
       gchar *load_inhibit = g_strdup ("");
       gchar *module_path;
 
-      g_setenv ("BABL_ERROR", "0.007", 0);
+      if (g_getenv ("BABL_ERROR") == NULL)
+        g_setenv ("BABL_ERROR", "0.007", 0);
 
       if (g_getenv ("GEGL_PATH"))
         {
@@ -186,7 +214,7 @@ gegl_post_parse_hook (GOptionContext *context,
         }
       else
         {
-          module_path  = g_strdup (PREFIX "/lib/gegl-1.0");
+          module_path = g_strdup (PREFIX "/lib/" GEGL_LIBRARY);
         }
 
       module_db = gegl_module_db_new (FALSE);
