@@ -3,7 +3,7 @@
  * GEGL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * GEGL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,9 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås
  */
@@ -25,9 +23,9 @@
 
 #include <gegl/gegl-types.h>
 
-#include <gegl/gegl-operation.h>
 #include <gegl/buffer/gegl-buffer.h>
 #include <gegl/module/geglmoduletypes.h>
 #include <gegl/module/geglmodule.h>
+#include <gegl/operation/gegl-operation.h>
 
 #endif  /* __GEGL_H__ */
