@@ -3,7 +3,7 @@
  * GEGL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * GEGL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,9 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås
  */
@@ -25,36 +23,39 @@
 #include <string.h>
 #include <limits.h>
 #include "gegl-types.h"
-#include "gegl-graph.h"
-#include "gegl-node.h"
-#include "gegl-operation.h"
-#include "gegl-pad.h"
-#include "gegl-color.h"
+#include "graph/gegl-node.h"
+#include "graph/gegl-pad.h"
+#include "property-types/gegl-color.h"
+#include "property-types/gegl-curve.h"
 #include "gegl-instrument.h"
 #include "gegl-xml.h"
-#include "gegl-paramspecs.h"
+#include "property-types/gegl-paramspecs.h"
+#include "operation/gegl-operation.h"
 
 #ifdef G_OS_WIN32
-#define realpath(a,b) _fullpath(b,a,_MAX_PATH)
+#define realpath(a, b)    _fullpath (b, a, _MAX_PATH)
 #endif
 
 typedef struct _ParseData ParseData;
 
-enum {
-  STATE_NONE=0,
+enum
+{
+  STATE_NONE = 0,
   STATE_TREE_NORMAL,
   STATE_TREE_FIRST_CHILD
 };
 
-struct _ParseData {
-  gint        state;
-  const gchar      *path_root;
-  GeglNode   *gegl;
-  gchar      *param; /*< the param we are setting (NULL when not in <param></param>) */
-  GeglNode   *iter;  /*< the iterator we're either connecting to input|aux of
+struct _ParseData
+{
+  gint         state;
+  const gchar *path_root;
+  GeglNode    *gegl;
+  gchar       *param;/*< the param we are setting (NULL when not in <param></param>) */
+  GeglNode    *iter; /*< the iterator we're either connecting to input|aux of
                          depending on context */
-  GList      *parent; /*< a stack of parents, as we are recursing into aux
+  GList       *parent;/*< a stack of parents, as we are recursing into aux
                          branches */
+  GeglCurve   *curve;/*< the curve whose points we are parsing */
 
   GHashTable *ids;
   GList      *refs;
@@ -100,15 +101,15 @@ static void param_set (ParseData   *pd,
     {
       GeglOperation *operation;
       GParamSpec    *paramspec;
-      set_clone_prop_as_well:
+set_clone_prop_as_well:
 
       operation = new->operation;
       paramspec = g_object_class_find_property (G_OBJECT_GET_CLASS (operation), param_name);
 
       if (!paramspec)
         {
-           g_warning ("property %s not found for %s",
-                      param_name, gegl_node_get_debug_name (new));
+          g_warning ("property %s not found for %s",
+                     param_name, gegl_node_get_debug_name (new));
         }
       else if (g_type_is_a (G_PARAM_SPEC_TYPE (paramspec),
                             GEGL_TYPE_PARAM_PATH))
@@ -141,7 +142,7 @@ static void param_set (ParseData   *pd,
       else if (paramspec->value_type == G_TYPE_FLOAT ||
                paramspec->value_type == G_TYPE_DOUBLE)
         {
-          gegl_node_set (new, param_name, atof (param_value), NULL);
+          gegl_node_set (new, param_name, g_ascii_strtod (param_value, NULL), NULL);
         }
       else if (paramspec->value_type == G_TYPE_STRING)
         {
@@ -157,7 +158,7 @@ static void param_set (ParseData   *pd,
               !strcmp (param_value, "Y") ||
               !strcmp (param_value, "1") ||
               !strcmp (param_value, "on")
-              )
+          )
             {
               gegl_node_set (new, param_name, TRUE, NULL);
             }
@@ -176,6 +177,16 @@ static void param_set (ParseData   *pd,
 
           g_object_unref (color);
         }
+      else if (paramspec->value_type == GEGL_TYPE_CURVE)
+        {
+	  if (pd->curve)
+	    {
+	      gegl_node_set (new, param_name, pd->curve, NULL);
+
+	      g_object_unref (pd->curve);
+	      pd->curve = NULL;
+	    }
+	}
       else
         {
           g_warning ("operation desired unknown parapspec type for %s",
@@ -191,13 +202,13 @@ static void start_element (GMarkupParseContext *context,
                            gpointer             user_data,
                            GError             **error)
 {
-  const gchar **a=attribute_names;
-  const gchar **v=attribute_values;
-  ParseData *pd;
+  const gchar **a = attribute_names;
+  const gchar **v = attribute_values;
+  ParseData    *pd;
 
   pd = user_data;
 
-  if (!strcmp (element_name, "gegl")||
+  if (!strcmp (element_name, "gegl") ||
       !strcmp (element_name, "image"))
     {
       GeglNode *new = g_object_new (GEGL_TYPE_NODE, NULL);
@@ -208,7 +219,7 @@ static void start_element (GMarkupParseContext *context,
       else
         {
         }
-      pd->state=STATE_TREE_NORMAL;
+      pd->state  = STATE_TREE_NORMAL;
       pd->parent = g_list_prepend (pd->parent, new);
 
       gegl_node_get_output_proxy (new, "output"); /* creates the pad if it doesn't exist */
@@ -228,15 +239,38 @@ static void start_element (GMarkupParseContext *context,
     {
       if (pd->param != NULL)
         g_warning ("eek, haven't cleared previous param");
-      g_assert (name2val (a,v, "name"));
+      g_assert (name2val (a, v, "name"));
       pd->param = g_strdup (name2val (a, v, "name"));
     }
-  else if (!strcmp (element_name, "link")||
-           !strcmp (element_name, "links")||
-           !strcmp (element_name, "stack")||
-           !strcmp (element_name, "launcher")||
-           !strcmp (element_name, "launchers")||
-           !strcmp (element_name, "source")||
+  else if (!strcmp (element_name, "curve"))
+    {
+      if (pd->curve != NULL)
+	g_warning ("we haven't cleared previous curve");
+      g_assert (name2val (a, v, "ymin"));
+      g_assert (name2val (a, v, "ymax"));
+      pd->curve = gegl_curve_new (g_ascii_strtod (name2val (a, v, "ymin"), NULL),
+				  g_ascii_strtod (name2val (a, v, "ymax"), NULL));
+    }
+  else if (!strcmp (element_name, "curve-point"))
+    {
+      if (!pd->curve)
+	g_warning ("curve not instantiated");
+      else
+        {
+	  g_assert (name2val (a, v, "x"));
+	  g_assert (name2val (a, v, "y"));
+
+	  gegl_curve_add_point (pd->curve,
+				g_ascii_strtod (name2val (a, v, "x"), NULL),
+				g_ascii_strtod (name2val (a, v, "y"), NULL));
+	}
+    }
+  else if (!strcmp (element_name, "link") ||
+           !strcmp (element_name, "links") ||
+           !strcmp (element_name, "stack") ||
+           !strcmp (element_name, "launcher") ||
+           !strcmp (element_name, "launchers") ||
+           !strcmp (element_name, "source") ||
            !strcmp (element_name, "destination"))
     {
       /* ignore */
@@ -268,9 +302,9 @@ static void start_element (GMarkupParseContext *context,
       while (*a)
         {
           param_set (pd, new, *a, *v);
-        a++;
-        v++;
-      }
+          a++;
+          v++;
+        }
 
       if (pd->state == STATE_TREE_FIRST_CHILD)
         {
@@ -282,8 +316,8 @@ static void start_element (GMarkupParseContext *context,
             gegl_node_connect_from (pd->iter, "input", new, "output");
         }
       pd->parent = g_list_prepend (pd->parent, new);
-      pd->state = STATE_TREE_FIRST_CHILD;
-      pd->iter = new;
+      pd->state  = STATE_TREE_FIRST_CHILD;
+      pd->iter   = new;
     }
 }
 
@@ -296,7 +330,7 @@ static void text (GMarkupParseContext *context,
   ParseData *pd;
 
   pd = user_data;
-  if (pd->param && pd->iter)
+  if (pd->param && pd->iter && !pd->curve)
     {
       param_set (pd, pd->iter, pd->param, text);
     }
@@ -312,7 +346,7 @@ static void end_element (GMarkupParseContext *context,
 
   pd = user_data;
 
-  if (!strcmp (element_name, "gegl")||
+  if (!strcmp (element_name, "gegl") ||
       !strcmp (element_name, "image"))
     {
       /*ignored*/
@@ -333,7 +367,7 @@ static void end_element (GMarkupParseContext *context,
           pd->iter = NULL;
         }
       pd->parent = g_list_remove (pd->parent, pd->parent->data);
-      pd->state = STATE_TREE_NORMAL;
+      pd->state  = STATE_TREE_NORMAL;
     }
   else if (!strcmp (element_name, "graph"))
     {
@@ -344,14 +378,20 @@ static void end_element (GMarkupParseContext *context,
       g_free (pd->param);
       pd->param = NULL;
     }
-  else if (!strcmp (element_name, "link")||
-           !strcmp (element_name, "links")||
-           !strcmp (element_name, "launcher")||
-           !strcmp (element_name, "launchers")||
-           !strcmp (element_name, "source")||
+  else if (!strcmp (element_name, "curve"))
+    {
+      g_assert (pd->param && pd->iter);
+      param_set (pd, pd->iter, pd->param, NULL);
+    }
+  else if (!strcmp (element_name, "link") ||
+           !strcmp (element_name, "links") ||
+           !strcmp (element_name, "launcher") ||
+           !strcmp (element_name, "launchers") ||
+           !strcmp (element_name, "source") ||
            !strcmp (element_name, "destination") ||
-           !strcmp (element_name, "stack")||
-           !strcmp (element_name, "params"))
+           !strcmp (element_name, "stack") ||
+           !strcmp (element_name, "params") ||
+	   !strcmp (element_name, "curve-point"))
     {
       /* ignore */
     }
@@ -361,7 +401,7 @@ static void end_element (GMarkupParseContext *context,
     {
       pd->iter   = pd->parent->data;
       pd->parent = g_list_remove (pd->parent, pd->parent->data);
-      pd->state = STATE_TREE_NORMAL;
+      pd->state  = STATE_TREE_NORMAL;
     }
 }
 
@@ -382,7 +422,7 @@ static void error (GMarkupParseContext *context,
              line_number, char_number, error->message);
 }
 
-static GMarkupParser parser={
+static GMarkupParser parser = {
   start_element,
   end_element,
   text,
@@ -393,7 +433,7 @@ static GMarkupParser parser={
 static void each_ref (gpointer value,
                       gpointer user_data)
 {
-  ParseData *pd=user_data;
+  ParseData *pd        = user_data;
   GeglNode  *dest_node = value;
   gchar     *ref;
   GeglNode  *source_node;
@@ -404,20 +444,21 @@ static void each_ref (gpointer value,
   gegl_node_connect_from (dest_node, "input", source_node, "output");
 }
 
-GeglNode *gegl_parse_xml (const gchar *xmldata,
-                          const gchar *path_root)
+GeglNode *gegl_node_new_from_xml (const gchar *xmldata,
+                                  const gchar *path_root)
 {
-  glong     time = gegl_ticks ();
-  GeglNode *ret;
+  glong                time = gegl_ticks ();
+  GeglNode            *ret;
   ParseData           *pd;
   GMarkupParseContext *context;
+
   pd = g_new0 (ParseData, 1);
 
   pd->ids = g_hash_table_new_full (g_str_hash,
                                    g_str_equal,
                                    g_free,
                                    NULL);
-  pd->refs = NULL;
+  pd->refs      = NULL;
   pd->path_root = path_root;
 
   context = g_markup_parse_context_new (&parser, 0, pd, NULL);
@@ -436,10 +477,42 @@ GeglNode *gegl_parse_xml (const gchar *xmldata,
   return ret;
 }
 
+GeglNode *
+gegl_node_new_from_file (const gchar   *path)
+{
+  GeglNode *node;
+  GError   *err = NULL;
+  gchar    *script;
+  gchar    *temp1;
+  gchar    *temp2;
+  gchar    *path_root;
+
+  g_assert (path);
+
+  temp1 = g_strdup (path);
+  temp2 = g_strdup (g_path_get_dirname (temp1));
+  path_root = g_strdup (realpath (temp2, NULL));
+
+  g_file_get_contents (path, &script, NULL, &err);
+  if (err != NULL)
+    {
+      g_warning ("Unable to read file: %s", err->message);
+      g_error_free (err);
+      return NULL;
+    }
+
+  node = gegl_node_new_from_xml (script, path_root);
+
+  g_free (temp1);
+  g_free (temp2);
+  g_free (path_root);
+  return node;
+}
+
 /****/
 
 
-#define ind do{gint i;for(i=0;i<indent;i++)g_string_append(ss->buf, " ");}while(0)
+#define ind    do { gint i; for (i = 0; i < indent; i++) g_string_append (ss->buf, " "); } while (0)
 
 typedef struct _SerializeState SerializeState;
 struct _SerializeState
@@ -474,13 +547,12 @@ static void tuple_old (GString     *buf,
       g_string_append (buf, key);
       g_string_append_c (buf, '=');
       g_string_append_c (buf, '\'');
-      for (p=text;*p;p++)
+      for (p = text; *p; p++)
         {
-          if (*p=='\n')
+          if (*p == '\n')
             g_string_append (buf, "&#10;");
           else
             g_string_append_c (buf, *p);
-
         }
       g_string_append_c (buf, '\'');
 
@@ -490,12 +562,12 @@ static void tuple_old (GString     *buf,
 
 static void encode_node_attributes_old (SerializeState *ss,
                                         GeglNode       *node,
-                                        const           gchar *id)
+                                        const gchar    *id)
 {
-  GParamSpec ** properties;
-  guint n_properties;
-  gchar *class;
-  gint i;
+  GParamSpec **properties;
+  guint        n_properties;
+  gchar       *class;
+  gint         i;
 
   properties = gegl_list_properties (gegl_node_get_operation (node), &n_properties);
 
@@ -506,13 +578,12 @@ static void encode_node_attributes_old (SerializeState *ss,
       g_free (class);
     }
 
-  for (i=0; i<n_properties; i++)
+  for (i = 0; i < n_properties; i++)
     {
       if (strcmp (properties[i]->name, "input") &&
           strcmp (properties[i]->name, "output") &&
           strcmp (properties[i]->name, "aux"))
         {
-
           if (g_type_is_a (G_PARAM_SPEC_TYPE (properties[i]),
                            GEGL_TYPE_PARAM_PATH))
             {
@@ -522,9 +593,9 @@ static void encode_node_attributes_old (SerializeState *ss,
               if (value)
                 {
                   if (ss->path_root &&
-                   !strncmp (ss->path_root, value, strlen (ss->path_root)))
+                      !strncmp (ss->path_root, value, strlen (ss->path_root)))
                     {
-                      tuple_old (ss->buf, properties[i]->name, &value[strlen(ss->path_root)+1]);
+                      tuple_old (ss->buf, properties[i]->name, &value[strlen (ss->path_root)]);
                     }
                   else
                     {
@@ -537,22 +608,22 @@ static void encode_node_attributes_old (SerializeState *ss,
           else if (properties[i]->value_type == G_TYPE_FLOAT)
             {
               gfloat value;
-              gchar str[64];
+              gchar  str[G_ASCII_DTOSTR_BUF_SIZE];
               gegl_node_get (node, properties[i]->name, &value, NULL);
-              sprintf (str, "%f", value);
+              g_ascii_dtostr (str, sizeof(str), value);
               tuple_old (ss->buf, properties[i]->name, str);
             }
           else if (properties[i]->value_type == G_TYPE_DOUBLE)
             {
               gdouble value;
-              gchar str[64];
+              gchar   str[G_ASCII_DTOSTR_BUF_SIZE];
               gegl_node_get (node, properties[i]->name, &value, NULL);
-              sprintf (str, "%f", value);
+              g_ascii_dtostr (str, sizeof(str), value);
               tuple_old (ss->buf, properties[i]->name, str);
             }
           else if (properties[i]->value_type == G_TYPE_INT)
             {
-              gint value;
+              gint  value;
               gchar str[64];
               gegl_node_get (node, properties[i]->name, &value, NULL);
               sprintf (str, "%i", value);
@@ -581,7 +652,7 @@ static void encode_node_attributes_old (SerializeState *ss,
           else if (properties[i]->value_type == GEGL_TYPE_COLOR)
             {
               GeglColor *color;
-              gchar *value;
+              gchar     *value;
               gegl_node_get (node, properties[i]->name, &color, NULL);
               g_object_get (color, "string", &value, NULL);
               g_object_unref (color);
@@ -591,12 +662,12 @@ static void encode_node_attributes_old (SerializeState *ss,
           else
             {
               g_warning ("%s: serialization of %s proeprties not implemented",
-                properties[i]->name, g_type_name (properties[i]->value_type));
+                         properties[i]->name, g_type_name (properties[i]->value_type));
             }
         }
     }
-    if (id != NULL)
-      tuple_old (ss->buf, "id", id);
+  if (id != NULL)
+    tuple_old (ss->buf, "id", id);
 
   g_free (properties);
 }
@@ -605,20 +676,19 @@ static void add_stack_old (SerializeState *ss,
                            gint            indent,
                            GeglNode       *head)
 {
-
   if (GEGL_IS_NODE (head))
     {
       GeglNode *iter = head;
 
       while (iter)
         {
-          GeglPad *input, *aux;
-          const gchar *id=NULL;
+          GeglPad     *input, *aux;
+          const gchar *id = NULL;
 
           input = gegl_node_get_pad (iter, "input");
-          aux = gegl_node_get_pad (iter, "aux");
+          aux   = gegl_node_get_pad (iter, "aux");
 
-          if (gegl_node_get_num_sinks (iter)>1)
+          if (gegl_node_get_num_sinks (iter) > 1)
             {
               const gchar *new_id = g_hash_table_lookup (ss->clones, iter);
               if (new_id)
@@ -634,7 +704,7 @@ static void add_stack_old (SerializeState *ss,
                   gchar temp_id[64];
                   sprintf (temp_id, "clone%i", ss->clone_count++);
                   id = g_strdup (temp_id);
-                  g_hash_table_insert (ss->clones, iter, (gchar*)id);
+                  g_hash_table_insert (ss->clones, iter, (gchar *) id);
                   /* the allocation is freed by the hash table */
                 }
             }
@@ -642,9 +712,9 @@ static void add_stack_old (SerializeState *ss,
           if (aux &&
               gegl_pad_get_connected_to (aux))
             {
-              GeglPad *source_pad;
+              GeglPad  *source_pad;
               GeglNode *source_node;
-              source_pad = gegl_pad_get_connected_to (aux);
+              source_pad  = gegl_pad_get_connected_to (aux);
               source_node = gegl_pad_get_node (source_pad);
               {
                 GeglNode *graph = g_object_get_data (G_OBJECT (source_node),
@@ -652,7 +722,7 @@ static void add_stack_old (SerializeState *ss,
                 if (graph)
                   source_node = graph;
               }
-              ind;if (ss->terse)
+              ind; if (ss->terse)
                 {
                   g_string_append (ss->buf, "<");
                   g_string_append (ss->buf, gegl_node_get_operation (iter));
@@ -663,9 +733,9 @@ static void add_stack_old (SerializeState *ss,
                 }
               encode_node_attributes_old (ss, iter, id);
               g_string_append (ss->buf, ">\n");
-              add_stack_old (ss, indent+4, source_node);
+              add_stack_old (ss, indent + 4, source_node);
 
-              ind;if (ss->terse)
+              ind; if (ss->terse)
                 {
                   g_string_append (ss->buf, "</");
                   g_string_append (ss->buf, gegl_node_get_operation (iter));
@@ -686,7 +756,7 @@ static void add_stack_old (SerializeState *ss,
                   if (strcmp (class, "nop") &&
                       strcmp (class, "clone"))
                     {
-                      ind;if (ss->terse)
+                      ind; if (ss->terse)
                         {
                           g_string_append (ss->buf, "<");
                           g_string_append (ss->buf, class);
@@ -694,7 +764,7 @@ static void add_stack_old (SerializeState *ss,
                       else
                         {
                           g_string_append (ss->buf, "<node");
-                      }
+                        }
 
                       encode_node_attributes_old (ss, iter, id);
                       g_string_append (ss->buf, "/>\n");
@@ -710,8 +780,8 @@ static void add_stack_old (SerializeState *ss,
               if (source_pad)
                 {
                   GeglNode *source_node = gegl_pad_get_node (source_pad);
-                  GeglNode *graph = g_object_get_data (G_OBJECT (source_node),
-                                                       "graph");
+                  GeglNode *graph       = g_object_get_data (G_OBJECT (source_node),
+                                                             "graph");
                   if (graph)
                     source_node = graph;
                   iter = source_node;
@@ -727,14 +797,15 @@ static void add_stack_old (SerializeState *ss,
 
 
 gchar *
-gegl_to_xml_old (GeglNode    *gegl,
-                 const gchar *path_root)
+gegl_node_to_xml_old (GeglNode    *gegl,
+                      const gchar *path_root)
 {
-  gchar *ret;
+  gchar          *ret;
   SerializeState *ss = g_new0 (SerializeState, 1);
-  ss->buf = g_string_new ("");
-  ss->clones = g_hash_table_new (NULL, NULL);
-  ss->terse = TRUE;
+
+  ss->buf       = g_string_new ("");
+  ss->clones    = g_hash_table_new (NULL, NULL);
+  ss->terse     = TRUE;
   ss->path_root = path_root;
 
   gegl = gegl_node_get_output_proxy (gegl, "output");
@@ -749,7 +820,7 @@ gegl_to_xml_old (GeglNode    *gegl,
   g_hash_table_foreach (ss->clones, free_clone_id, NULL);
   g_hash_table_destroy (ss->clones);
 
-  ret=g_string_free (ss->buf, FALSE);
+  ret = g_string_free (ss->buf, FALSE);
   g_free (ss);
   return ret;
 }
@@ -770,13 +841,12 @@ static void xml_attr (GString     *buf,
       g_string_append (buf, key);
       g_string_append_c (buf, '=');
       g_string_append_c (buf, '\'');
-      for (p=text;*p;p++)
+      for (p = text; *p; p++)
         {
-          if (*p=='\n')
+          if (*p == '\n')
             g_string_append (buf, "&#10;");
           else
             g_string_append_c (buf, *p);
-
         }
       g_string_append_c (buf, '\'');
 
@@ -784,6 +854,45 @@ static void xml_attr (GString     *buf,
     }
 }
 
+static void xml_param_start (SerializeState *ss,
+			     gint            indent,
+			     const gchar    *key)
+{
+  g_assert (key);
+  ind; g_string_append (ss->buf, "<param name='");
+  g_string_append (ss->buf, key);
+  g_string_append (ss->buf, "'>");
+}
+
+static void xml_param_text (SerializeState *ss,
+			    const gchar    *value)
+{
+  gchar *text;
+  /*gchar *p;*/
+
+  g_assert (value);
+
+  /* Why isn't this used here??? */
+  text = g_markup_escape_text (value, -1);
+
+  g_string_append (ss->buf, value);
+  /*for (p=text;*p;p++)
+    {
+    if (*p=='\n')
+    g_string_append (ss->buf, "&#10;");
+    else
+    g_string_append_c (ss->buf, *p);
+    
+    }*/
+
+  g_free (text);
+}
+
+static void xml_param_end (SerializeState *ss)
+{
+  g_string_append (ss->buf, "</param>\n");
+}
+
 static void xml_param (SerializeState *ss,
                        gint            indent,
                        const gchar    *key,
@@ -792,41 +901,68 @@ static void xml_param (SerializeState *ss,
   g_assert (key);
 
   if (value)
+  {
+    xml_param_start (ss, indent, key);
+    xml_param_text (ss, value);
+    xml_param_end (ss);
+  }
+}
+
+static void xml_curve_point (SerializeState *ss,
+			     gint            indent,
+			     gfloat          x,
+			     gfloat          y)
+{
+  gchar str[64];
+  ind; g_string_append (ss->buf, "<curve-point x='");
+  g_ascii_dtostr (str, sizeof(str), x);
+  g_string_append (ss->buf, str);
+  g_string_append (ss->buf, "' y='");
+  g_ascii_dtostr (str, sizeof(str), y);
+  g_string_append (ss->buf, str);
+  g_string_append (ss->buf, "'/>\n");
+}
+
+static void xml_curve (SerializeState *ss,
+		       gint            indent,
+		       GeglCurve      *curve)
+{
+  gchar str[G_ASCII_DTOSTR_BUF_SIZE];
+  gdouble min_y, max_y;
+  guint num_points = gegl_curve_num_points (curve);
+  guint i;
+
+  gegl_curve_get_y_bounds (curve, &min_y, &max_y);
+
+  ind; g_string_append (ss->buf, "<curve ymin='");
+  g_ascii_dtostr (str, sizeof(str), min_y);
+  g_string_append (ss->buf, str);
+  g_string_append (ss->buf, "' ymax='");
+  g_ascii_dtostr (str, sizeof(str), max_y);
+  g_string_append (ss->buf, str);
+  g_string_append (ss->buf, "'>\n");
+  for (i = 0; i < num_points; ++i)
     {
-      gchar *text = g_markup_escape_text (value, -1);
-      /*gchar *p;*/
-
-      ind;g_string_append (ss->buf, "<param name='");
-      g_string_append (ss->buf, key);
-      g_string_append (ss->buf, "'>");
-      g_string_append (ss->buf, value);
-      /*for (p=text;*p;p++)
-        {
-          if (*p=='\n')
-            g_string_append (ss->buf, "&#10;");
-          else
-            g_string_append_c (ss->buf, *p);
-
-        }*/
-      g_string_append (ss->buf, "</param>\n");
-
-      g_free (text);
+      gdouble x, y;
+      gegl_curve_get_point (curve, i, &x, &y);
+      xml_curve_point (ss, indent + 2, x, y);
     }
+  ind; g_string_append (ss->buf, "</curve>\n");
 }
 
 static void serialize_properties (SerializeState *ss,
                                   gint            indent,
                                   GeglNode       *node)
 {
-  GParamSpec ** properties;
-  guint n_properties;
-  gboolean got_a_param = FALSE;
-  gint i;
+  GParamSpec **properties;
+  guint        n_properties;
+  gboolean     got_a_param = FALSE;
+  gint         i;
 
   properties = gegl_list_properties (gegl_node_get_operation (node), &n_properties);
 
 
-  for (i=0; i<n_properties; i++)
+  for (i = 0; i < n_properties; i++)
     {
       if (strcmp (properties[i]->name, "input") &&
           strcmp (properties[i]->name, "output") &&
@@ -847,13 +983,13 @@ static void serialize_properties (SerializeState *ss,
               if (value)
                 {
                   if (ss->path_root &&
-                   !strncmp (ss->path_root, value, strlen (ss->path_root)))
+                      !strncmp (ss->path_root, value, strlen (ss->path_root)))
                     {
-                      xml_param (ss, indent+2, properties[i]->name, &value[strlen(ss->path_root)+1]);
+                      xml_param (ss, indent + 2, properties[i]->name, &value[strlen (ss->path_root) + 1]);
                     }
                   else
                     {
-                      xml_param (ss, indent+2, properties[i]->name, value);
+                      xml_param (ss, indent + 2, properties[i]->name, value);
                     }
                 }
 
@@ -862,26 +998,26 @@ static void serialize_properties (SerializeState *ss,
           else if (properties[i]->value_type == G_TYPE_FLOAT)
             {
               gfloat value;
-              gchar str[64];
+              gchar  str[G_ASCII_DTOSTR_BUF_SIZE];
               gegl_node_get (node, properties[i]->name, &value, NULL);
-              sprintf (str, "%f", value);
-              xml_param (ss, indent+2, properties[i]->name, str);
+              g_ascii_dtostr (str, sizeof(str), value);
+              xml_param (ss, indent + 2, properties[i]->name, str);
             }
           else if (properties[i]->value_type == G_TYPE_DOUBLE)
             {
               gdouble value;
-              gchar str[64];
+              gchar   str[G_ASCII_DTOSTR_BUF_SIZE];
               gegl_node_get (node, properties[i]->name, &value, NULL);
-              sprintf (str, "%f", value);
-              xml_param (ss, indent+2, properties[i]->name, str);
+              g_ascii_dtostr (str, sizeof(str), value);
+              xml_param (ss, indent + 2, properties[i]->name, str);
             }
           else if (properties[i]->value_type == G_TYPE_INT)
             {
-              gint value;
+              gint  value;
               gchar str[64];
               gegl_node_get (node, properties[i]->name, &value, NULL);
               sprintf (str, "%i", value);
-              xml_param (ss, indent+2, properties[i]->name, str);
+              xml_param (ss, indent + 2, properties[i]->name, str);
             }
           else if (properties[i]->value_type == G_TYPE_BOOLEAN)
             {
@@ -889,42 +1025,54 @@ static void serialize_properties (SerializeState *ss,
               gegl_node_get (node, properties[i]->name, &value, NULL);
               if (value)
                 {
-                  xml_param (ss, indent+2, properties[i]->name, "true");
+                  xml_param (ss, indent + 2, properties[i]->name, "true");
                 }
               else
                 {
-                  xml_param (ss, indent+2, properties[i]->name, "false");
+                  xml_param (ss, indent + 2, properties[i]->name, "false");
                 }
             }
           else if (properties[i]->value_type == G_TYPE_STRING)
             {
               gchar *value;
               gegl_node_get (node, properties[i]->name, &value, NULL);
-              xml_param (ss, indent+2, properties[i]->name, value);
+              xml_param (ss, indent + 2, properties[i]->name, value);
               g_free (value);
             }
           else if (properties[i]->value_type == GEGL_TYPE_COLOR)
             {
               GeglColor *color;
-              gchar *value;
+              gchar     *value;
               gegl_node_get (node, properties[i]->name, &color, NULL);
               g_object_get (color, "string", &value, NULL);
               g_object_unref (color);
-              xml_param (ss, indent+2, properties[i]->name, value);
+              xml_param (ss, indent + 2, properties[i]->name, value);
               g_free (value);
             }
+	  else if (properties[i]->value_type == GEGL_TYPE_CURVE)
+	    {
+	      GeglCurve *curve;
+	      guint num_points;
+	      gegl_node_get (node, properties[i]->name, &curve, NULL);
+	      num_points = gegl_curve_num_points (curve);
+	      xml_param_start (ss, indent + 2, properties[i]->name);
+	      g_string_append (ss->buf, "\n");
+	      xml_curve (ss, indent + 4, curve);
+	      indent += 2; ind; indent -= 2; xml_param_end (ss);
+	      g_object_unref (curve);
+	    }
           else
             {
-              g_warning ("%s: serialization of %s proeprties not implemented",
-                properties[i]->name, g_type_name (properties[i]->value_type));
+              g_warning ("%s: serialization of %s properties not implemented",
+                         properties[i]->name, g_type_name (properties[i]->value_type));
             }
         }
     }
 
-    if (got_a_param)
-      {
-        ind; g_string_append (ss->buf, "</params>\n");
-      }
+  if (got_a_param)
+    {
+      ind; g_string_append (ss->buf, "</params>\n");
+    }
   g_free (properties);
 }
 
@@ -932,35 +1080,47 @@ static void serialize_layer (SerializeState *ss,
                              gint            indent,
                              GeglNode       *layer)
 {
-  gchar *name;
-  gchar *src;
-  gchar *composite_op;
+  gchar  *name;
+  gchar  *src;
+  gchar  *composite_op;
   gdouble x;
   gdouble y;
   gdouble opacity;
 
   gegl_node_get (layer, "name", &name,
-                        "src", &src,
-                        "x", &x,
-                        "y", &y,
-                        "opacity", &opacity,
-                        "composite_op", &composite_op,
-                        NULL);
+                 "src", &src,
+                 "x", &x,
+                 "y", &y,
+                 "opacity", &opacity,
+                 "composite_op", &composite_op,
+                 NULL);
   ind; g_string_append (ss->buf, "<layer");
   if (name[0])
     g_string_append_printf (ss->buf, " name='%s'", name);
-  if (x!=0.0)
-    g_string_append_printf (ss->buf, " x='%f'", x);
-  if (y!=0.0)
-    g_string_append_printf (ss->buf, " y='%f'", y);
-  if (opacity!=1.0)
-    g_string_append_printf (ss->buf, " opacity='%f'", opacity);
+  if (x != 0.0)
+    {
+      gchar str[G_ASCII_DTOSTR_BUF_SIZE];
+      g_ascii_dtostr (str, sizeof(str), x);
+      g_string_append_printf (ss->buf, " x='%s'", str);
+    }
+  if (y != 0.0)
+    {
+      gchar str[G_ASCII_DTOSTR_BUF_SIZE];
+      g_ascii_dtostr (str, sizeof(str), y);
+      g_string_append_printf (ss->buf, " y='%s'", str);
+    }
+  if (opacity != 1.0)
+    {
+      gchar str[G_ASCII_DTOSTR_BUF_SIZE];
+      g_ascii_dtostr (str, sizeof(str), opacity);
+      g_string_append_printf (ss->buf, " opacity='%s'", str);
+    }
   if (src[0])
     {
       if (ss->path_root &&
           !strncmp (ss->path_root, src, strlen (ss->path_root)))
         {
-          g_string_append_printf (ss->buf, " src='%s'", &src[strlen(ss->path_root)+1]);
+          g_string_append_printf (ss->buf, " src='%s'", &src[strlen (ss->path_root) + 1]);
         }
       else
         {
@@ -968,7 +1128,6 @@ static void serialize_layer (SerializeState *ss,
         }
     }
   g_string_append (ss->buf, "/>\n");
-
 }
 
 static void add_stack (SerializeState *ss,
@@ -976,7 +1135,7 @@ static void add_stack (SerializeState *ss,
                        GeglNode       *head)
 {
   /*ind; g_string_append (ss->buf, "<stack>\n");
-  indent+=2;*/
+     indent+=2;*/
 
   if (GEGL_IS_NODE (head))
     {
@@ -984,15 +1143,15 @@ static void add_stack (SerializeState *ss,
 
       while (iter)
         {
-          GeglPad *input, *aux;
-          const gchar *id=NULL;
-          gchar *class;
+          GeglPad     *input, *aux;
+          const gchar *id = NULL;
+          gchar       *class;
           gegl_node_get (iter, "operation", &class, NULL);
 
           input = gegl_node_get_pad (iter, "input");
-          aux = gegl_node_get_pad (iter, "aux");
+          aux   = gegl_node_get_pad (iter, "aux");
 
-          if (gegl_node_get_num_sinks (iter)>1)
+          if (gegl_node_get_num_sinks (iter) > 1)
             {
               const gchar *new_id = g_hash_table_lookup (ss->clones, iter);
               if (new_id)
@@ -1008,7 +1167,7 @@ static void add_stack (SerializeState *ss,
                   gchar temp_id[64];
                   sprintf (temp_id, "clone%i", ss->clone_count++);
                   id = g_strdup (temp_id);
-                  g_hash_table_insert (ss->clones, iter, (gchar*)id);
+                  g_hash_table_insert (ss->clones, iter, (gchar *) id);
                   /* the allocation is freed by the hash table */
                 }
             }
@@ -1022,9 +1181,9 @@ static void add_stack (SerializeState *ss,
               if (aux &&
                   gegl_pad_get_connected_to (aux))
                 {
-                  GeglPad *source_pad;
+                  GeglPad  *source_pad;
                   GeglNode *source_node;
-                  source_pad = gegl_pad_get_connected_to (aux);
+                  source_pad  = gegl_pad_get_connected_to (aux);
                   source_node = gegl_pad_get_node (source_pad);
                   {
                     GeglNode *graph = g_object_get_data (G_OBJECT (source_node),
@@ -1032,14 +1191,14 @@ static void add_stack (SerializeState *ss,
                     if (graph)
                       source_node = graph;
                   }
-                  ind;g_string_append (ss->buf, "<node");
+                  ind; g_string_append (ss->buf, "<node");
 
                   {
                     gchar *class;
                     gchar *name;
                     gegl_node_get (iter, "operation", &class,
-                                         "name", &name,
-                                         NULL);
+                                   "name", &name,
+                                   NULL);
                     if (name[0])
                       {
                         xml_attr (ss->buf, "name", name);
@@ -1052,27 +1211,26 @@ static void add_stack (SerializeState *ss,
                   }
 
                   g_string_append (ss->buf, ">\n");
-                  serialize_properties (ss, indent+4, iter);
-                  add_stack (ss, indent+4, source_node);
+                  serialize_properties (ss, indent + 4, iter);
+                  add_stack (ss, indent + 4, source_node);
 
-                  ind;g_string_append (ss->buf, "</node>\n");
+                  ind; g_string_append (ss->buf, "</node>\n");
                 }
               else
                 {
-
                   if (class)
                     {
-                      if (strcmp (class, "nop") && 
+                      if (strcmp (class, "nop") &&
                           strcmp (class, "clone"))
                         {
-                          ind;g_string_append (ss->buf, "<node");
+                          ind; g_string_append (ss->buf, "<node");
 
                           {
                             gchar *class;
                             gchar *name;
                             gegl_node_get (iter, "operation", &class,
-                                                 "name", &name,
-                                                 NULL);
+                                           "name", &name,
+                                           NULL);
                             if (name[0])
                               {
                                 xml_attr (ss->buf, "name", name);
@@ -1085,8 +1243,8 @@ static void add_stack (SerializeState *ss,
                           }
 
                           g_string_append (ss->buf, ">\n");
-                          serialize_properties (ss, indent+4, iter);
-                          ind;g_string_append (ss->buf, "</node>\n");
+                          serialize_properties (ss, indent + 4, iter);
+                          ind; g_string_append (ss->buf, "</node>\n");
                         }
                     }
                 }
@@ -1099,8 +1257,8 @@ static void add_stack (SerializeState *ss,
               if (source_pad)
                 {
                   GeglNode *source_node = gegl_pad_get_node (source_pad);
-                  GeglNode *graph = g_object_get_data (G_OBJECT (source_node),
-                                                       "graph");
+                  GeglNode *graph       = g_object_get_data (G_OBJECT (source_node),
+                                                             "graph");
                   if (graph)
                     source_node = graph;
                   iter = source_node;
@@ -1117,18 +1275,19 @@ static void add_stack (SerializeState *ss,
         }
     }
   /*indent-=2;
-  ind; g_string_append (ss->buf, "<stack>\n");*/
+     ind; g_string_append (ss->buf, "<stack>\n");*/
 }
 
 gchar *
-gegl_to_xml (GeglNode    *gegl,
-             const gchar *path_root)
+gegl_node_to_xml (GeglNode    *gegl,
+                  const gchar *path_root)
 {
-  gchar *ret;
+  gchar          *ret;
   SerializeState *ss = g_new0 (SerializeState, 1);
-  ss->buf = g_string_new ("");
-  ss->clones = g_hash_table_new (NULL, NULL);
-  ss->terse = FALSE;
+
+  ss->buf       = g_string_new ("");
+  ss->clones    = g_hash_table_new (NULL, NULL);
+  ss->terse     = FALSE;
   ss->path_root = path_root;
 
   gegl = gegl_node_get_output_proxy (gegl, "output");
@@ -1143,7 +1302,7 @@ gegl_to_xml (GeglNode    *gegl,
   g_hash_table_foreach (ss->clones, free_clone_id, NULL);
   g_hash_table_destroy (ss->clones);
 
-  ret=g_string_free (ss->buf, FALSE);
+  ret = g_string_free (ss->buf, FALSE);
   g_free (ss);
   return ret;
 }
