@@ -3,7 +3,7 @@
  * GEGL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * GEGL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,9 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * 2000-2007 © Calvin Williamson, Øyvind Kolås.
  */
@@ -22,19 +20,21 @@
 #define __GEGL_H__
 
 #include <glib-object.h>
-
+#include <babl/babl.h>
 
 /***
  * The GEGL API:
  *
- * This document is both a tutorial and the reference for the C API of GEGL.
- * The concepts covered in the tutotrial should be applicable using other
- * languages as well.
+ * This document is both a tutorial and a reference for the C API of GEGL.
+ * The concepts covered in this reference should also be applicable when
+ * using other languages.
  *
- * The core API of GEGL isn't frozen yet and feedback regarding it's use as
+ * The core API of GEGL isn't frozen yet and feedback regarding its use as
  * well as the clarity of this documentation is most welcome.
  */
 
+G_BEGIN_DECLS
+
 /***
  * Introduction:
  *
@@ -43,13 +43,10 @@
  * pads which can be connected. By connecting these nodes in chains a set of
  * image operation filters and combinators can be applied to the image data.
  *
- * To make GEGL process data you request a rectangular region of an node's
- * output pad to be rendered into a provided linear buffer, of any (supported
- * by babl) pixel format. GEGL is using information about the rectangular
- * bounding-boxes for regions that are needed to compute the requested result.
- * For many forms of processing it should thus be efficient to ask for only a
- * subregion to be recomputed. This fits well with being able to back an api
- * that requests rectangular regions to be repainted.
+ * To make GEGL process data you request a rectangular region of a node's
+ * output pad to be rendered into a provided linear buffer of any (supported
+ * by babl) pixel format. GEGL uses information provided by the nodes to
+ * determine the smallest buffers needed at each stage of processing.
  */
 
 /***
@@ -79,11 +76,14 @@
  * initialize everything needed to operate GEGL and parses some
  * standard command line options.  @argc and @argv are adjusted
  * accordingly so your own code will never see those standard
- * arguments.
+ * arguments. gegl_init() will call g_thread_init(), unless you, or
+ * some other code already has initialized gthread.
  *
  * Note that there is an alternative way to initialize GEGL: if you
  * are calling g_option_context_parse() with the option group returned
- * by #gegl_get_option_group(), you don't have to call #gegl_init().
+ * by #gegl_get_option_group(), you don't have to call #gegl_init() but
+ * you have to call g_thread_init() before any glib or glib dependant code
+ * yourself.
  **/
 void           gegl_init                 (gint          *argc,
                                           gchar       ***argv);
@@ -107,6 +107,46 @@ GOptionGroup * gegl_get_option_group     (void);
 void           gegl_exit                 (void);
 
 
+/***
+ * Available operations:
+ * Gegl provides means to check for available processing operations that
+ * can be used with nodes using #gegl_list_operations and for a specified
+ * op give a list of properties with #gegl_list_properties.
+ */
+
+/**
+ * gegl_list_operations:
+ * @n_operations_p: return location for number of operations.
+ *
+ * Returns an alphabetically sorted array of available operation names. The
+ * list should be freed with g_free after use.
+ * ---
+ * gchar **operations;
+ * guint   n_operations;
+ * gint i;
+ *
+ * operations = gegl_list_operations (&n_operations);
+ * g_print ("Available operations:\n");
+ * for (i=0; i < n_operations; i++)
+ *   {
+ *     g_print ("\t%s\n", operations[i]);
+ *   }
+ * g_free (operations);
+ */
+gchar        **gegl_list_operations         (guint *n_operations_p);
+
+
+/**
+ * gegl_list_properties:
+ * @operation_type: the name of the operation type we want to query to properties of.
+ * @n_properties_p: return location for number of properties.
+ *
+ * Returns an allocated array of #GParamSpecs describing the properties
+ * of the operation available when a node has operation_type set.
+ */
+GParamSpec** gegl_list_properties           (const gchar   *operation_type,
+                                             guint         *n_properties_p);
+
 /***
  * GeglNode:
  *
@@ -266,7 +306,7 @@ gboolean      gegl_node_disconnect       (GeglNode      *node,
                                           const gchar   *input_pad);
 
 /***
- * Setting properties:
+ * Properties:
  *
  * Properties can be set either when creating the node with
  * #gegl_node_new_child as well as later when changing the initial
@@ -298,6 +338,25 @@ void          gegl_node_set              (GeglNode      *node,
                                           const gchar   *first_property_name,
                                           ...) G_GNUC_NULL_TERMINATED;
 
+/**
+ * gegl_node_get:
+ * @node: a #GeglNode
+ * @first_property_name: name of the first property to get.
+ * @...: return location for the first property, followed optionally by more
+ * name/value pairs, followed by NULL.
+ *
+ * Gets properties of a #GeglNode.
+ * ---
+ * double level;
+ * char  *path;
+ *
+ * gegl_node_get (png_save, "path", &path, NULL);
+ * gegl_node_get (threshold, "level", &level, NULL);
+ */
+void          gegl_node_get              (GeglNode      *node,
+                                          const gchar   *first_property_name,
+                                          ...) G_GNUC_NULL_TERMINATED;
+
 
 /***
  * Processing:
@@ -306,7 +365,8 @@ void          gegl_node_set              (GeglNode      *node,
  * query any node providing output for a rectangular region to be rendered
  * using #gegl_node_blit, or you use #gegl_node_process on a sink node (A
  * display node, an image file writer or similar). To do iterative processing
- * you need to use a #GeglProcessor see #gegl_processor_work for a code sample.
+ * you need to use a #GeglProcessor. See #gegl_processor_work for a code
+ * sample.
  */
 
 #ifndef GEGL_INTERNAL
@@ -318,16 +378,22 @@ typedef enum
 } GeglBlitFlags;
 #endif
 
+#ifndef GEGL_AUTO_ROWSTRIDE
+#define GEGL_AUTO_ROWSTRIDE 0
+#endif
+
 /**
  * gegl_node_blit:
  * @node: a #GeglNode
- * @roi: the upper left coordinates to render, and the width/height of the
- * destination buffer.
  * @scale: the scale to render at 1.0 is default, other values changes the
  * width/height of the sampled region.
+ * @roi: the rectangle to render from the node, the coordinate system used is
+ * coordinates after scale has been applied.
  * @format: the #BablFormat desired.
- * @rowstride: rowstride in bytes (currently ignored)
- * @destination_buf: a memory buffer large enough to contain the data.
+ * @destination_buf: a memory buffer large enough to contain the data, can be
+ * left as NULL when forcing a rendering of a region.
+ * @rowstride: rowstride in bytes, or GEGL_AUTO_ROWSTRIDE to compute the
+ * rowstride based on the width and bytes per pixel for the specified format.
  * @flags: an or'ed combination of GEGL_BLIT_DEFAULT, GEGL_BLIT_CACHE and
  * GEGL_BLIT_DIRTY. if cache is enabled, a cache will be set up for subsequent
  * requests of image data from this node. By passing in GEGL_BLIT_DIRTY the
@@ -337,11 +403,11 @@ typedef enum
  * Render a rectangular region from a node.
  */
 void          gegl_node_blit             (GeglNode      *node,
-                                          GeglRectangle *roi,
                                           gdouble        scale,
-                                          void          *format,
+                                          GeglRectangle *roi,
+                                          Babl          *format,
+                                          gpointer       destination_buf,
                                           gint           rowstride,
-                                          gpointer      *destination_buf,
                                           GeglBlitFlags  flags);
 
 /**
@@ -349,8 +415,11 @@ void          gegl_node_blit             (GeglNode      *node,
  * @sink_node: a #GeglNode without outputs.
  *
  * Render a composition. This can be used for instance on a node with a "png-save"
- * operation to render all neccesary data, and make it be written to file. The
- * function is blocking for a non blocking way of doing the same see #GeglProcessor.
+ * operation to render all neccesary data, and make it be written to file. This
+ * function wraps the usage of a GeglProcessor in a single blocking function
+ * call. If you need a non-blocking operation, then make a direct use of
+ * #gegl_processor_work. See #GeglProcessor.
+ *
  * ---
  * GeglNode      *gegl;
  * GeglRectangle  roi;
@@ -381,74 +450,6 @@ void          gegl_node_blit             (GeglNode      *node,
 void          gegl_node_process          (GeglNode      *sink_node);
 
 
-/**
- * GeglProcessor:
- *
- * A #GeglProcessor, is a worker that can be used for background rendering
- * of regions in a node's cache. Or for processing a sink node. For most
- * non GUI tasks using #gegl_node_blit and #gegl_node_process directly
- * should be sufficient. See #gegl_processor_work for a code sample.
- *
- */
-#ifndef GEGL_INTERNAL
-GType gegl_processor_get_type  (void) G_GNUC_CONST;
-typedef struct _GeglProcessor      GeglProcessor;
-#define GEGL_TYPE_PROCESSOR  (gegl_processor_get_type())
-#define GEGL_PROCESSOR(obj)  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_PROCESSOR, GeglProcessor))
-#endif
-
-/**
- * gegl_node_new_processor:
- * @node: a #GeglNode
- * @rectangle: the #GeglRectangle to work on or NULL to work on all available
- * data.
- *
- * Returns a new #GeglProcessor.
- */
-GeglProcessor *gegl_node_new_processor      (GeglNode      *node,
-                                             GeglRectangle *rectangle);
-
-/**
- * gegl_processor_set_rectangle:
- * @processor: a #GeglProcessor
- * @rectangle: the new #GeglRectangle the processor shold work on or NULL
- * to make it work on all data in the buffer.
- *
- * Change the rectangle a #GeglProcessor is working on.
- */
-void           gegl_processor_set_rectangle (GeglProcessor *processor,
-                                             GeglRectangle *rectangle);
-
-
-/**
- * gegl_processor_work:
- * @processor: a #GeglProcessor
- * @progress: a location to store the (estimated) percentage complete.
- *
- * Do an iteration of work for the processor.
- *
- * Returns TRUE if there is more work to be done.
- *
- * ---
- * GeglProcessor *processor = gegl_node_new_processor (node, &roi);
- * double         progress;
- *
- * while (gegl_processor_work (processor, &progress))
- *   g_warning ("%f%% complete", progress);
- * gegl_processor_destroy (processor);
- */
-gboolean       gegl_processor_work          (GeglProcessor *processor,
-                                             gdouble       *progress);
-
-
-/**
- * gegl_processor_destroy:
- * @processor: a #GeglProcessor
- *
- * Frees up resources used by a processing handle.
- */
-void           gegl_processor_destroy       (GeglProcessor *processor);
-
 
 /***
  * State queries:
@@ -460,39 +461,6 @@ void           gegl_processor_destroy       (GeglProcessor *processor);
  * retrieve the values of named properties using #gegl_node_get.
  */
 
-/**
- * gegl_list_operations:
- * @n_operations_p: return location for number of operations.
- *
- * Returns an alphabetically sorted array of available operation names. The
- * list should be freed with g_free after use.
- * --
- * gchar **operations;
- * guint   n_operations;
- * gint i;
- *
- * operations = gegl_list_operations (&n_operations);
- * g_print ("Available operations:\n");
- * for (i=0; i < n_operations; i++)
- *   {
- *     g_print ("\t%s\n", operations[i]);
- *   }
- * g_free (operations);
- */
-gchar        **gegl_list_operations         (guint *n_operations_p);
-
-
-/**
- * gegl_list_properties:
- * @operation_type: the name of the operation type we want to query to properties of.
- * @n_properties_p: return location for number of properties.
- *
- * Returns an allocated array of #GParamSpecs describing the properties
- * of the operation currently set for a node. The returned array should
- * be freed with g_free.
- */
-GParamSpec** gegl_list_properties           (const gchar   *operation_type,
-                                             guint         *n_properties_p);
 
 
 /**
@@ -524,24 +492,6 @@ GParamSpec  * gegl_node_find_property    (GeglNode      *node,
                                           const gchar   *property_name);
 
 
-/**
- * gegl_node_get:
- * @node: a #GeglNode
- * @first_property_name: name of the first property to get.
- * @...: return location for the first property, followed optionally by more
- * name/value pairs, followed by NULL.
- *
- * Gets properties of a #GeglNode.
- * ---
- * double level;
- * char  *path;
- *
- * gegl_node_get (png_save, "path", &path, NULL);
- * gegl_node_get (threshold, "level", &level, NULL);
- */
-void          gegl_node_get              (GeglNode      *node,
-                                          const gchar   *first_property_name,
-                                          ...) G_GNUC_NULL_TERMINATED;
 
 /**
  * gegl_node_get_bounding_box:
@@ -634,129 +584,11 @@ GeglNode    * gegl_node_get_producer     (GeglNode      *node,
 
 
 /***
- * XML:
- * The XML format used by GEGL is not stable and should not be relied on
- * for anything but testing purposes yet.
- */
-
-/**
- * gegl_parse_xml:
- * @xmldata: a \0 terminated string containing XML data to be parsed.
- * @path_root: a file system path that relative paths in the XML will be
- * resolved in relation to.
+ * Binding conveniences:
  *
- * The #GeglNode returned contains the graph described by the tree of stacks
- * in the XML document. The tree is connected to the "output" pad of the
- * returned node and thus can be used directly for processing.
- *
- * Returns a GeglNode containing the parsed XML as a subgraph.
- */
-GeglNode    * gegl_parse_xml             (const gchar   *xmldata,
-                                          const gchar   *path_root);
-
-/**
- * gegl_to_xml:
- * @node: a #GeglNode
- * @path_root: filesystem path to construct relative paths from.
- *
- * Returns a freshly allocated \0 terminated string containing a XML
- * serialization of the composition produced by a node (and thus also
- * the nodes contributing data to the specified node). To export a
- * gegl graph, connect the internal output node to an output proxy(see
- * #gegl_node_get_output_proxy.
- */
-gchar       * gegl_to_xml                (GeglNode      *node,
-                                          const gchar   *path_root);
-
-#ifndef GEGL_INTERNAL
-
-/***
- * GeglRectangle:
- *
- * GeglRectangles are used in #gegl_node_get_bounding_box and #gegl_node_blit
- * for specifying rectangles.
- *
- * </p><pre>struct GeglRectangle
- * {
- *   gint x;
- *   gint y;
- *   gint width;
- *   gint height;
- * };</pre><p>
- *
- */
-struct _GeglRectangle
-{
-  gint x;
-  gint y;
-  gint width;
-  gint height;
-};
-
-/***
- * GeglColor:
- *
- * GeglColor is used for properties that use a gegl color, use #gegl_color_new
- * with a NULL string to create a new blank one, gegl_colors are destroyed
- * with g_object_unref when they no longer are needed.
- *
- * The colors used by gegls are described in a format similar to CSS. The
- * textstring "rgb(1.0,1.0,1.0)" signifies opaque white and
- * "rgba(1.0,0.0,0.0,0.75)" is a 75% opaque red. Hexadecimal forms like #RRGGBB
- * and #RRGGBBAA are also supported.
- */
-typedef struct _GeglColor     GeglColor;
-GType gegl_color_get_type (void) G_GNUC_CONST;
-#define GEGL_TYPE_COLOR (gegl_color_get_type())
-#define GEGL_COLOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_COLOR, GeglColor))
-#endif
-
-/**
- * gegl_color_new:
- * @string: an CSS style color string.
- *
- * Returns a #GeglColor object suitable for use with #gegl_node_set.
- */
-GeglColor   * gegl_color_new             (const gchar   *string);
-
-/**
- * gegl_color_get_rgba:
- * @color: a #GeglColor
- * @r: return location for red component.
- * @g: return location for green component.
- * @b: return location for blue component.
- * @a: return location for alpha component.
- *
- * Retrieve RGB component values from a #GeglColor.
- */
-void          gegl_color_get_rgba        (GeglColor     *color,
-                                          gfloat        *r,
-                                          gfloat        *g,
-                                          gfloat        *b,
-                                          gfloat        *a);
-
-/**
- * gegl_color_set_rgba:
- * @color: a #GeglColor
- * @r: new value for red component
- * @g: new value for green component
- * @b: new value for blue component
- * @a: new value for alpha component
- *
- * Retrieve RGB component values from a GeglColor.
- */
-void          gegl_color_set_rgba        (GeglColor     *color,
-                                          gfloat         r,
-                                          gfloat         g,
-                                          gfloat         b,
-                                          gfloat         a);
-
-/***
- * Bindings conveniences:
- *
- * The following functions are mostly included to make it easier
- * to create language bindings. The varargs versions most often
- * lead to more readable C code.
+ * The following functions are mostly included to make it easier to create
+ * language bindings for the nodes. The varargs versions will in most cases
+ * lead to both more efficient and readable code from C.
  */
 
 /**
@@ -765,11 +597,15 @@ void          gegl_color_set_rgba        (GeglColor     *color,
  * @operation: the type of node to create.
  *
  * Creates a new processing node that performs the specified operation.
+ * All properties of the operation will have their default values. This
+ * is included as an addiiton to #gegl_node_new_child in the public API to have
+ * a non varargs entry point for bindings as well as sometimes simpler more
+ * readable code.
  *
  * Returns a newly created node. The node will be destroyed by the parent.
  * Calling g_object_unref on a node will cause the node to be dropped by the
  * parent. (You may also add additional references using
- * g_object_ref/g_objecr_unref, but in general relying on the parents reference
+ * g_object_ref/g_object_unref, but in general relying on the parents reference
  * counting is easiest.)
  */
 
@@ -804,9 +640,263 @@ void          gegl_node_set_property     (GeglNode      *node,
                                           const gchar   *property_name,
                                           const GValue  *value);
 
+/***
+ * XML:
+ * The XML format used by GEGL is not stable and should not be relied on
+ * for anything but testing purposes yet.
+ */
+
+/**
+ * gegl_node_new_from_xml:
+ * @xmldata: a \0 terminated string containing XML data to be parsed.
+ * @path_root: a file system path that relative paths in the XML will be
+ * resolved in relation to.
+ *
+ * The #GeglNode returned contains the graph described by the tree of stacks
+ * in the XML document. The tree is connected to the "output" pad of the
+ * returned node and thus can be used directly for processing.
+ *
+ * Returns a GeglNode containing the parsed XML as a subgraph.
+ */
+GeglNode    * gegl_node_new_from_xml     (const gchar   *xmldata,
+                                          const gchar   *path_root);
+
+/**
+ * gegl_node_new_from_file:
+ * @path: the path to a file on the local file system to be parsed.
+ *
+ * The #GeglNode returned contains the graph described by the tree of stacks
+ * in the XML document. The tree is connected to the "output" pad of the
+ * returned node and thus can be used directly for processing.
+ *
+ * Returns a GeglNode containing the parsed XML as a subgraph.
+ */
+GeglNode    * gegl_node_new_from_file    (const gchar   *path);
+
+/**
+ * gegl_node_to_xml:
+ * @node: a #GeglNode
+ * @path_root: filesystem path to construct relative paths from.
+ *
+ * Returns a freshly allocated \0 terminated string containing a XML
+ * serialization of the composition produced by a node (and thus also
+ * the nodes contributing data to the specified node). To export a
+ * gegl graph, connect the internal output node to an output proxy (see
+ * #gegl_node_get_output_proxy.) and use the proxy node as the basis
+ * for the serialization.
+ */
+gchar       * gegl_node_to_xml           (GeglNode      *node,
+                                          const gchar   *path_root);
+/***
+ * GeglProcessor:
+ *
+ * A #GeglProcessor, is a worker that can be used for background rendering
+ * of regions in a node's cache. Or for processing a sink node. For most
+ * non GUI tasks using #gegl_node_blit and #gegl_node_process directly
+ * should be sufficient. See #gegl_processor_work for a code sample.
+ *
+ */
+#ifndef GEGL_INTERNAL
+GType gegl_processor_get_type  (void) G_GNUC_CONST;
+typedef struct _GeglProcessor      GeglProcessor;
+#define GEGL_TYPE_PROCESSOR  (gegl_processor_get_type())
+#define GEGL_PROCESSOR(obj)  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_PROCESSOR, GeglProcessor))
+#endif
+
+/**
+ * gegl_node_new_processor:
+ * @node: a #GeglNode
+ * @rectangle: the #GeglRectangle to work on or NULL to work on all available
+ * data.
+ *
+ * Returns a new #GeglProcessor.
+ */
+GeglProcessor *gegl_node_new_processor      (GeglNode      *node,
+                                             GeglRectangle *rectangle);
+
+/**
+ * gegl_processor_set_rectangle:
+ * @processor: a #GeglProcessor
+ * @rectangle: the new #GeglRectangle the processor shold work on or NULL
+ * to make it work on all data in the buffer.
+ *
+ * Change the rectangle a #GeglProcessor is working on.
+ */
+void           gegl_processor_set_rectangle (GeglProcessor *processor,
+                                             GeglRectangle *rectangle);
+
+
+/**
+ * gegl_processor_work:
+ * @processor: a #GeglProcessor
+ * @progress: a location to store the (estimated) percentage complete.
+ *
+ * Do an iteration of work for the processor.
+ *
+ * Returns TRUE if there is more work to be done.
+ *
+ * ---
+ * GeglProcessor *processor = gegl_node_new_processor (node, &roi);
+ * double         progress;
+ *
+ * while (gegl_processor_work (processor, &progress))
+ *   g_warning ("%f%% complete", progress);
+ * gegl_processor_destroy (processor);
+ */
+gboolean       gegl_processor_work          (GeglProcessor *processor,
+                                             gdouble       *progress);
+
+
+/**
+ * gegl_processor_destroy:
+ * @processor: a #GeglProcessor
+ *
+ * Frees up resources used by a processing handle.
+ */
+void           gegl_processor_destroy       (GeglProcessor *processor);
+
+
+
+#ifndef GEGL_INTERNAL
+
+
+/***
+ * GeglColor:
+ *
+ * GeglColor is used for properties that use a gegl color, use #gegl_color_new
+ * with a NULL string to create a new blank one, gegl_colors are destroyed
+ * with g_object_unref when they no longer are needed.
+ *
+ * The colors used by gegls are described in a format similar to CSS. The
+ * textstring "rgb(1.0,1.0,1.0)" signifies opaque white and
+ * "rgba(1.0,0.0,0.0,0.75)" is a 75% opaque red. Hexadecimal forms like #RRGGBB
+ * and #RRGGBBAA are also supported.
+ */
+typedef struct _GeglColor     GeglColor;
+GType gegl_color_get_type (void) G_GNUC_CONST;
+#define GEGL_TYPE_COLOR (gegl_color_get_type())
+#define GEGL_COLOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_COLOR, GeglColor))
+#endif
+
+/**
+ * gegl_color_new:
+ * @string: a CSS-style color string.
+ *
+ * Returns a #GeglColor object suitable for use with #gegl_node_set.
+ */
+GeglColor   * gegl_color_new                (const gchar   *string);
+
+/**
+ * gegl_color_get_rgba:
+ * @color: a #GeglColor
+ * @r: return location for red component.
+ * @g: return location for green component.
+ * @b: return location for blue component.
+ * @a: return location for alpha component.
+ *
+ * Retrieve RGB component values from a #GeglColor.
+ */
+void          gegl_color_get_rgba           (GeglColor     *color,
+                                             gfloat        *r,
+                                             gfloat        *g,
+                                             gfloat        *b,
+                                             gfloat        *a);
+
+/**
+ * gegl_color_set_rgba:
+ * @color: a #GeglColor
+ * @r: new value for red component
+ * @g: new value for green component
+ * @b: new value for blue component
+ * @a: new value for alpha component
+ *
+ * Retrieve RGB component values from a GeglColor.
+ */
+void          gegl_color_set_rgba           (GeglColor     *color,
+                                             gfloat         r,
+                                             gfloat         g,
+                                             gfloat         b,
+                                             gfloat         a);
+
+#ifndef GEGL_INTERNAL
+/***
+ * GeglRectangle:
+ *
+ * GeglRectangles are used in #gegl_node_get_bounding_box and #gegl_node_blit
+ * for specifying rectangles.
+ *
+ * </p><pre>struct GeglRectangle
+ * {
+ *   gint x;
+ *   gint y;
+ *   gint width;
+ *   gint height;
+ * };</pre><p>
+ *
+ */
+struct _GeglRectangle
+{
+  gint x;
+  gint y;
+  gint width;
+  gint height;
+};
+
+/***
+ * GeglCurve:
+ *
+ * Documentation and function signatures left out of documentation until
+ * API is frozen.
+ */
+typedef struct _GeglCurve       GeglCurve;
+GType        gegl_curve_get_type            (void) G_GNUC_CONST;
+#define GEGL_TYPE_CURVE            (gegl_curve_get_type ())
+#define GEGL_CURVE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_CURVE, GeglCurve))
+#endif
+
+GeglCurve*   gegl_curve_new                 (gdouble         y_min,
+                                             gdouble         y_max);
+
+GeglCurve*   gegl_curve_default_curve       (void) G_GNUC_CONST;
+
+void         gegl_curve_get_y_bounds        (GeglCurve      *self,
+                                             gdouble        *min_y,
+                                             gdouble        *max_y);
+
+guint        gegl_curve_add_point           (GeglCurve      *self,
+                                             gdouble         x,
+                                             gdouble         y);
+
+void         gegl_curve_remove_point_index  (GeglCurve      *self,
+                                             guint           index);
+
+void         gegl_curve_get_point           (GeglCurve      *self,
+                                             guint           index,
+                                             gdouble        *x,
+                                             gdouble        *y);
+
+void         gegl_curve_set_point           (GeglCurve      *self,
+                                             guint           index,
+                                             gdouble         x,
+                                             gdouble         y);
+ 
+guint        gegl_curve_num_points          (GeglCurve      *self);
+
+gdouble     gegl_curve_calc_value           (GeglCurve      *self,
+                                             gdouble         x);
+
+void         gegl_curve_calc_values         (GeglCurve      *self,
+                                             gdouble         x_min,
+                                             gdouble         x_max,
+                                             guint           num_samples,
+                                             gdouble        *xs,
+                                             gdouble        *ys);
+
+G_END_DECLS
+
 /*** this is just here to trick the parser.
  */
-#include "gegl/gegl-paramspecs.h"
+#include "gegl/property-types/gegl-paramspecs.h"
 #include <babl/babl.h>
 
 #endif  /* __GEGL_H__ */
