@@ -3,7 +3,7 @@
  * GEGL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * GEGL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,17 +11,17 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Philip Lafleur
  */
 
+/* TODO: reenable different samplers (through the sampling mechanism
+ *       of GeglBuffer intitially) */
 /* TODO: only calculate pixels inside transformed polygon */
 /* TODO: should hard edges always be used when only scaling? */
-/* TODO: solve rounding problems in rect calculation if possible
-         (see scale_*) */
+/* TODO: make rect calculations depend on the sampling kernel of the
+ *       interpolation filter used */
 
 #include <string.h>
 #include <math.h>
@@ -31,10 +31,7 @@
 #include "affine.h"
 #include "module.h"
 #include "matrix.h"
-#include "nearest.h"
-#include "linear.h"
-#include "interpolate-lanczos.h"
-#include "interpolate-cubic.h"
+#include "buffer/gegl-sampler.h"
 
 enum
 {
@@ -63,11 +60,13 @@ static gboolean      is_composite_node    (OpAffine      *affine);
 static void          get_source_matrix    (OpAffine      *affine,
                                            Matrix3        output);
 static GeglRectangle get_defined_region   (GeglOperation *op);
-static gboolean      calc_source_regions  (GeglOperation *op,
-                                           gpointer       context_id);
-static GeglRectangle get_affected_region  (GeglOperation *operation,
+static GeglRectangle compute_affected_region  (GeglOperation *operation,
                                            const gchar   *input_pad,
                                            GeglRectangle  region);
+static GeglRectangle compute_input_request (GeglOperation *self,
+                                           const gchar   *input_pad,
+                                           GeglRectangle *region);
+
 static gboolean      process              (GeglOperation *op,
                                            gpointer       context_id);
 static GeglNode    * detect               (GeglOperation *operation,
@@ -118,6 +117,17 @@ op_affine_get_type (void)
 
 /* ************************* */
 
+static void
+prepare (GeglOperation *operation,
+         gpointer       context_id)
+{
+  Babl *format = babl_format ("RaGaBaA float");
+
+  /*gegl_operation_set_format (operation, "input", format);
+  gegl_operation_set_format (operation, "aux", format); XXX(not used yet) */
+  gegl_operation_set_format (operation, "output", format);
+}
+
 static void
 op_affine_class_init (OpAffineClass *klass)
 {
@@ -128,11 +138,12 @@ op_affine_class_init (OpAffineClass *klass)
   gobject_class->set_property = set_property;
   gobject_class->get_property = get_property;
 
-  op_class->get_affected_region = get_affected_region;
+  op_class->compute_affected_region = compute_affected_region;
   op_class->get_defined_region  = get_defined_region;
-  op_class->calc_source_regions = calc_source_regions;
+  op_class->compute_input_request = compute_input_request;
   op_class->detect = detect;
   op_class->categories = "transform";
+  op_class->prepare = prepare;
 
   filter_class->process = process;
 
@@ -435,9 +446,9 @@ detect (GeglOperation *operation,
   return gegl_operation_detect (source_node->operation, need_points[0], need_points[1]);
 }
 
-static gboolean
-calc_source_regions (GeglOperation *op,
-                     gpointer       context_id)
+static GeglRectangle compute_input_request (GeglOperation *op,
+                                            const gchar   *input_pad,
+                                            GeglRectangle *region)
 {
   OpAffine      *affine = (OpAffine *) op;
   Matrix3        inverse;
@@ -446,7 +457,7 @@ calc_source_regions (GeglOperation *op,
   gdouble        need_points [8];
   gint           i;
 
-  requested_rect = *(gegl_operation_get_requested_region (op, context_id));
+  requested_rect = *region;
 
   matrix3_copy (inverse, affine->matrix);
   matrix3_invert (inverse);
@@ -454,8 +465,7 @@ calc_source_regions (GeglOperation *op,
   if (is_intermediate_node (affine) ||
       matrix3_is_identity (inverse))
     {
-      gegl_operation_set_source_region (op, context_id, "input", &requested_rect);
-      return TRUE;
+      return requested_rect;
     }
 
   need_points [0] = requested_rect.x;
@@ -494,12 +504,11 @@ calc_source_regions (GeglOperation *op,
         }
     }
 
-  gegl_operation_set_source_region (op, context_id, "input", &need_rect);
-  return TRUE;
+  return need_rect;
 }
 
 static GeglRectangle
-get_affected_region (GeglOperation *op,
+compute_affected_region (GeglOperation *op,
                      const gchar   *input_pad,
                      GeglRectangle  region)
 {
@@ -568,95 +577,140 @@ get_affected_region (GeglOperation *op,
 
 }
 
+void
+affine_generic (GeglBuffer        *dest,
+                GeglBuffer        *src,
+                Matrix3            matrix,
+                GeglInterpolation  interpolation)
+{
+  const GeglRectangle *dest_extent;
+  gint                  x, y;
+  gfloat               *dest_buf,
+                       *dest_ptr;
+  Matrix3               inverse;
+  gdouble               u_start,
+                        v_start,
+                        u_float,
+                        v_float;
+
+  Babl                 *format; 
+  
+  gint                  dest_pixels;
+
+  format = babl_format ("RaGaBaA float");
+
+  /* XXX: fast paths as existing in files in the same dir as affine.c
+   *      should probably be hooked in here, and bailing out before using
+   *      the generic code.
+   */
+  g_object_get (dest, "pixels", &dest_pixels, NULL);
+  dest_extent = gegl_buffer_get_extent (dest);
+
+  dest_buf = g_new (gfloat, dest_pixels * 4);
+
+  matrix3_copy (inverse, matrix);
+  matrix3_invert (inverse);
+
+  u_start = inverse[0][0] * dest_extent->x + inverse[0][1] * dest_extent->y
+            + inverse[0][2];
+  v_start = inverse[1][0] * dest_extent->x + inverse[1][1] * dest_extent->y
+            + inverse[1][2];
+
+  /* correct rounding on e.g. negative scaling (is this sound?) */
+  if (inverse [0][0] < 0.)
+    u_start -= .001;
+  if (inverse [1][1] < 0.)
+    v_start -= .001;
+
+  if (src->sampler)
+    {
+      gegl_sampler_prepare (src->sampler);
+    }
+
+  for (dest_ptr = dest_buf, y = dest_extent->height; y--;)
+    {
+      u_float = u_start;
+      v_float = v_start;
+
+      for (x = dest_extent->width; x--;)
+        {
+          gfloat  pix[4];
+          gegl_buffer_sample (src, u_float, v_float, 1.0, pix, format, interpolation);
+
+          *dest_ptr++ = pix[0];
+          *dest_ptr++ = pix[1];
+          *dest_ptr++ = pix[2];
+          *dest_ptr++ = pix[3];
+
+          u_float += inverse [0][0];
+          v_float += inverse [1][0];
+        }
+      u_start += inverse [0][1];
+      v_start += inverse [1][1];
+    }
+  gegl_buffer_sample_cleanup (src);
+  gegl_buffer_set (dest, NULL, format, dest_buf);
+  g_free (dest_buf);
+}
+
+
 
 static gboolean
-process (GeglOperation *op,
+process (GeglOperation *operation,
          gpointer       context_id)
 {
-  OpAffine      *affine = (OpAffine *) op;
-  GeglBuffer    *filter_input;
+  OpAffine      *affine = (OpAffine *) operation;
+  GeglBuffer    *input;
   GeglBuffer    *output;
-  GeglRectangle *result = gegl_operation_result_rect (op, context_id);
+  GeglRectangle *result;
+ 
+  result = gegl_operation_result_rect (operation, context_id);
 
-  filter_input = GEGL_BUFFER (gegl_operation_get_data (op, context_id, "input"));
+  input = gegl_operation_get_source (operation, context_id, "input");
+  output = gegl_operation_get_target (operation, context_id, "output");
+
+  /*g_warning ("%i,%i %ix%i | %i,%i %ix%i | %i,%i  %ix%i",
+     input->x, input->y, input->width, input->height,
+     output->x, output->y, output->width, output->height,
+     result->x, result->y, result->width, result->height);*/
 
   if (is_intermediate_node (affine) ||
       matrix3_is_identity (affine->matrix))
-    output = g_object_new (GEGL_TYPE_BUFFER,
-        "source", filter_input,
-        "x",      result->x,
-        "y",      result->y,
-        "width",  result->width ,
-        "height", result->height,
-        NULL);
+    {
+      /* XXX: make the copying smarter, by perhaps even COW sharing
+       * of tiles (in the gegl_buffer_copy code)
+       */
+      gegl_buffer_copy (input, NULL, output, NULL);
+    }
+#if 0  /* XXX: port to use newer APIs, perhaps not possible to achieve COW,
+                but should still be faster than a full resampling when it
+                is not needed */
   else if (matrix3_is_translate (affine->matrix) &&
            (! strcasecmp (affine->filter, "nearest") ||
             (affine->matrix [0][2] == (gint) affine->matrix [0][2] &&
              affine->matrix [1][2] == (gint) affine->matrix [1][2])))
-    output = g_object_new (GEGL_TYPE_BUFFER,
-                           "source",      filter_input,
-                           "x",           result->x,
-                           "y",           result->y,
-                           "width",       result->width ,
-                           "height",      result->height,
-                           "shift-x",     (gint) - affine->matrix [0][2],
-                           "shift-y",     (gint) - affine->matrix [1][2],
-                           "abyss-width", -1, /* use source's abyss */
-                           NULL);
-  else
     {
       output = g_object_new (GEGL_TYPE_BUFFER,
-          "format", babl_format ("RaGaBaA float"),
-          "x",      result->x,
-          "y",      result->y,
-          "width",  result->width ,
-          "height", result->height,
-          NULL);
-
-      if (! strcasecmp (affine->filter, "linear"))
-        {
-          GeglBuffer *input;
-
-          if (affine->hard_edges)
-            input = g_object_new (GEGL_TYPE_BUFFER,
-              "source", filter_input,
-              "x",      filter_input->x,
-              "y",      filter_input->y,
-              "width",  filter_input->width + 1,
-              "height", filter_input->height + 1,
-              NULL);
-          else
-            input = g_object_new (GEGL_TYPE_BUFFER,
-                "source", filter_input,
-                "x",      filter_input->x - 1,
-                "y",      filter_input->y - 1,
-                "width",  filter_input->width + 2,
-                "height", filter_input->height + 2,
-                NULL);
-
-          if (matrix3_is_scale (affine->matrix))
-            scale_linear (output, input, affine->matrix, affine->hard_edges);
-          else
-            affine_linear (output, input, affine->matrix, affine->hard_edges);
-
-          g_object_unref (input);
-        }
-      else if (!strcasecmp (affine->filter, "lanczos"))
-        if (matrix3_is_scale (affine->matrix))
-          scale_lanczos (output, filter_input, affine->matrix, affine->lanczos_width);
-        else
-          affine_lanczos (output, filter_input, affine->matrix, affine->lanczos_width);
-      else if (!strcasecmp (affine->filter, "cubic"))
-        if (matrix3_is_scale (affine->matrix))
-          scale_cubic (output, filter_input, affine->matrix);
-        else
-          affine_cubic (output, filter_input, affine->matrix);
-      else
-          if (matrix3_is_scale (affine->matrix))
-            scale_nearest (output, filter_input, affine->matrix);
-          else
-            affine_nearest (output, filter_input, affine->matrix);
+                             "provider",    input,
+                             "x",           result->x,
+                             "y",           result->y,
+                             "width",       result->width ,
+                             "height",      result->height,
+                             "shift-x",     (gint) - affine->matrix [0][2],
+                             "shift-y",     (gint) - affine->matrix [1][2],
+                             "abyss-width", -1, /* use source's abyss */
+                             NULL);
+      /* fast path for affine translate with integer coordinates */
+      gegl_operation_set_data (operation, context_id, "output", G_OBJECT (output));
+      return TRUE;
     }
-  gegl_operation_set_data (op, context_id, "output", G_OBJECT (output));
+#endif
+  else
+    {
+      /* XXX: add back more samplers */
+      affine_generic (output, input, affine->matrix, gegl_buffer_interpolation_from_string (
+         affine->filter));
+    }
+  g_object_unref (input);
   return TRUE;
 }
