@@ -3,7 +3,7 @@
  * GEGL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * GEGL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,15 +11,13 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
  */
 #if GEGL_CHANT_PROPERTIES
 
-gegl_chant_double (value, 0.0, 4.0, 0.5,
+gegl_chant_double (value, -10.0, 10.0, 0.5,
    "Global threshold level (used when there is no auxiliary input buffer).")
 
 #else
@@ -29,13 +27,15 @@ gegl_chant_double (value, 0.0, 4.0, 0.5,
 #define GEGL_CHANT_DESCRIPTION     "Thresholds the image to white/black based on either the global value set in the value property, or per pixel from the aux input."
 #define GEGL_CHANT_SELF            "threshold.c"
 #define GEGL_CHANT_CATEGORIES      "color"
-#define GEGL_CHANT_INIT
+#define GEGL_CHANT_PREPARE
 #include "gegl-chant.h"
 
-static void init (GeglChantOperation *self)
+static void prepare (GeglOperation *operation,
+                     gpointer       context_id)
 {
-  GEGL_OPERATION_POINT_COMPOSER (self)->format = babl_format ("YA float");
-  GEGL_OPERATION_POINT_COMPOSER (self)->aux_format = babl_format ("Y float");
+  gegl_operation_set_format (operation, "input", babl_format ("YA float"));
+  gegl_operation_set_format (operation, "aux", babl_format ("Y float"));
+  gegl_operation_set_format (operation, "output", babl_format ("YA float"));
 }
 
 static gboolean
@@ -45,7 +45,7 @@ process (GeglOperation *op,
          void          *out_buf,
          glong          n_pixels)
 {
-  gint i;
+  glong   i;
 
   gfloat *in = in_buf;
   gfloat *out = out_buf;
