@@ -3,7 +3,7 @@
  * GEGL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * GEGL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,9 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
  */
@@ -34,13 +32,16 @@
 #define GEGL_CHANT_DESCRIPTION  "Allows changing the whitepoint and blackpoint of an image."
 
 #define GEGL_CHANT_CATEGORIES   "color"
-#define GEGL_CHANT_INIT
+#define GEGL_CHANT_PREPARE
 
 #include "gegl-chant.h"
 
-static void init (GeglChantOperation *self)
+static void prepare (GeglOperation *operation,
+                     gpointer       context_id)
 {
-  GEGL_OPERATION_POINT_FILTER (self)->format = babl_format ("Y'CbCrA float");
+  Babl *format = babl_format ("Y'CbCrA float");
+  gegl_operation_set_format (operation, "input", format);
+  gegl_operation_set_format (operation, "output", format);
 }
 
 /* GeglOperationPointFilter gives us a linear buffer to operate on
@@ -58,7 +59,7 @@ process (GeglOperation *op,
   gfloat              a_scale;
   gfloat              b_base;
   gfloat              b_scale;
-  gint                i;
+  glong               i;
 
   self = GEGL_CHANT_OPERATION (op);
   pixel = in_buf;
