@@ -3,7 +3,7 @@
  * GEGL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * GEGL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,9 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
  */
@@ -51,22 +49,14 @@ process (GeglOperation *operation,
 {
   GeglBuffer          *input;
   GeglBuffer          *output;
-  GeglChantOperation  *crop;
+  GeglChantOperation  *crop = GEGL_CHANT_OPERATION (operation);
+  GeglRectangle        extent = {crop->x, crop->y, crop->width, crop->height};
   
-  crop   = GEGL_CHANT_OPERATION (operation);
   input = GEGL_BUFFER (gegl_operation_get_data (operation, context_id, "input"));
 
-
   g_assert (input);
-  g_assert (gegl_buffer_get_format (input));
 
-  output = g_object_new (GEGL_TYPE_BUFFER,
-                         "source",       input,
-                         "x",      (int)crop->x,
-                         "y",      (int)crop->y,
-                         "width",  (int)crop->width,
-                         "height", (int)crop->height,
-                         NULL);
+  output = gegl_buffer_create_sub_buffer (input, &extent);
   gegl_operation_set_data (operation, context_id, "output", G_OBJECT (output));
   return  TRUE;
 }
@@ -93,17 +83,8 @@ get_defined_region (GeglOperation *operation)
   return result;
 }
 
-static gboolean
-calc_source_regions (GeglOperation *self,
-                     gpointer       context_id)
-{
-  gegl_operation_set_source_region (self, context_id, "input",
-                                    gegl_operation_get_requested_region (self, context_id));
-  return TRUE;
-}
-
 static GeglRectangle
-get_affected_region (GeglOperation *operation,
+compute_affected_region (GeglOperation *operation,
                      const gchar   *input_pad,
                      GeglRectangle  region)
 {
@@ -118,9 +99,9 @@ get_affected_region (GeglOperation *operation,
 
 static void class_init (GeglOperationClass *operation_class)
 {
-  operation_class->get_affected_region = get_affected_region;
+  operation_class->compute_affected_region = compute_affected_region;
   operation_class->get_defined_region = get_defined_region;
-  operation_class->calc_source_regions = calc_source_regions;
+  operation_class->no_cache = TRUE;
 }
 
 #endif
