@@ -3,7 +3,7 @@
  * GEGL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * GEGL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,9 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
  */
@@ -50,7 +48,7 @@ detect (GeglOperation *operation,
 {
   GeglNode *input_node;
 
-  const gchar *name = gegl_object_get_name (GEGL_OBJECT (operation->node));
+  const gchar *name = gegl_node_get_name (operation->node);
   if (name && !strcmp (name, "proxynop-output"))
     {
       GeglNode *graph;
@@ -70,9 +68,7 @@ detect (GeglOperation *operation,
 static void class_init (GeglOperationClass *operation_class)
 {
   operation_class->detect = detect;
+  operation_class->no_cache = TRUE;
 }
 
-
-
-
 #endif
