@@ -3,7 +3,7 @@
  * GEGL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * GEGL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,9 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
  */
@@ -24,12 +22,12 @@ gegl_chant_pointer(buf, "Buffer")
 
 #else
 
-#define GEGL_CHANT_SOURCE
 #define GEGL_CHANT_NAME           introspect
-#define GEGL_CHANT_DESCRIPTION    "GEGL graph vizualizer."
-
 #define GEGL_CHANT_SELF           "introspect.c"
+#define GEGL_CHANT_DESCRIPTION    "GEGL graph vizualizer."
 #define GEGL_CHANT_CATEGORIES     "render"
+
+#define GEGL_CHANT_SOURCE
 #include "gegl-chant.h"
 
 
@@ -68,6 +66,8 @@ process (GeglOperation *operation,
       g_free (dot);
     }
 
+    /* FIXME: copy behavior from magick-load to fix this op */
+
 
     {
       GeglNode *gegl = gegl_node_new ();
@@ -76,13 +76,9 @@ process (GeglOperation *operation,
       GeglRectangle defined;
 
       defined = gegl_node_get_bounding_box (png_load);
-      self->buf = g_object_new (GEGL_TYPE_BUFFER,
-                          "format", babl_format ("R'G'B' u8"),
-                          "x",      defined.x,
-                          "y",      defined.y,
-                          "width",  defined.width ,
-                          "height", defined.height ,
-                          NULL);
+
+      self->buf = gegl_buffer_new (&defined, babl_format ("R'G'B' u8"));
+
       buffer_save = gegl_node_new_child (gegl, "operation", "save-buffer", "buffer", self->buf, NULL);
       gegl_node_link_many (png_load, buffer_save, NULL);
 
@@ -110,8 +106,7 @@ get_defined_region (GeglOperation *operation)
   if (self->buf)
     {
       GeglBuffer *buffer = GEGL_BUFFER (self->buf);
-      gegl_rectangle_set (&result, buffer->x, buffer->y,
-                              buffer->width, buffer->height);
+      result = *gegl_buffer_get_extent (buffer);
     }
   return result;
 }
