@@ -3,7 +3,7 @@
  * GEGL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * GEGL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,9 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
  */
@@ -24,23 +22,21 @@ gegl_chant_double (radius, 0.0, 50.0, 4.0,
 
 #else
 
-#define GEGL_CHANT_FILTER
 #define GEGL_CHANT_NAME            kuwahara_max
-#define GEGL_CHANT_DESCRIPTION     "Edge preserving max filter"
 #define GEGL_CHANT_SELF            "kuwahara-max.c"
+#define GEGL_CHANT_DESCRIPTION     "Edge preserving max filter"
 #define GEGL_CHANT_CATEGORIES      "misc"
-#define GEGL_CHANT_CLASS_INIT
+
+#define GEGL_CHANT_AREA_FILTER
+
 #include "gegl-chant.h"
+#include <math.h>
 
 static void
 kuwahara (GeglBuffer *src,
           GeglBuffer *dst,
           gint        radius);
 
-static GeglRectangle get_source_rect (GeglOperation *self,
-                                      gpointer       context_id);
-
-#include <stdio.h>
 
 static gboolean
 process (GeglOperation *operation,
@@ -58,49 +54,18 @@ process (GeglOperation *operation,
   input = GEGL_BUFFER (gegl_operation_get_data (operation, context_id, "input"));
     {
       GeglRectangle   *result = gegl_operation_result_rect (operation, context_id);
-      GeglRectangle    work   = *result;
       GeglBuffer      *temp_in;
+      GeglRectangle    compute  = gegl_operation_compute_input_request (operation, "inputt", gegl_operation_need_rect (operation, context_id));
 
-      work.x-=self->radius;
-      work.y-=self->radius;
-      work.width +=self->radius*2;
-      work.height +=self->radius*2;
+      temp_in = gegl_buffer_create_sub_buffer (input, &compute);
+      output = gegl_buffer_new (&compute, babl_format ("RGBA float"));
 
-      if (result->width ==0 ||
-          result->height==0)
-        {
-          output = g_object_ref (input);
-        }
-      else
-        {
-          temp_in = g_object_new (GEGL_TYPE_BUFFER,
-                                 "source", input,
-                                 "x",      work.x,
-                                 "y",      work.y,
-                                 "width",  work.width ,
-                                 "height", work.height ,
-                                 NULL);
-
-          output = g_object_new (GEGL_TYPE_BUFFER,
-                                 "format", babl_format ("RGBA float"),
-                                 "x",      work.x,
-                                 "y",      work.y,
-                                 "width",  work.width ,
-                                 "height", work.height ,
-                                 NULL);
-
-          kuwahara (temp_in, output, self->radius);
-          g_object_unref (temp_in);
-        }
+      kuwahara (temp_in, output, self->radius);
+      g_object_unref (temp_in);
+      
 
       {
-        GeglBuffer *cropped = g_object_new (GEGL_TYPE_BUFFER,
-                                              "source", output,
-                                              "x",      result->x,
-                                              "y",      result->y,
-                                              "width",  result->width ,
-                                              "height", result->height,
-                                              NULL);
+        GeglBuffer *cropped = gegl_buffer_create_sub_buffer (output, result);
         gegl_operation_set_data (operation, context_id, "output", G_OBJECT (cropped));
         g_object_unref (output);
       }
@@ -168,14 +133,14 @@ kuwahara (GeglBuffer *src,
   gfloat *src_buf;
   gfloat *dst_buf;
 
-  src_buf = g_malloc0 (src->width * src->height * 4 * 4);
-  dst_buf = g_malloc0 (dst->width * dst->height * 4 * 4);
+  src_buf = g_malloc0 (gegl_buffer_get_pixel_count (src) * 4 * 4);
+  dst_buf = g_malloc0 (gegl_buffer_get_pixel_count (dst) * 4 * 4);
 
-  gegl_buffer_get (src, NULL, 1.0, babl_format ("RGBA float"), src_buf);
+  gegl_buffer_get (src, 1.0, NULL, babl_format ("RGBA float"), src_buf, GEGL_AUTO_ROWSTRIDE);
 
   offset = 0;
-  for (v=0; v<dst->height; v++)
-    for (u=0; u<dst->width; u++)
+  for (v=0; v<gegl_buffer_get_height (dst); v++)
+    for (u=0; u<gegl_buffer_get_width (dst); u++)
       {
         gint component;
 
@@ -188,8 +153,8 @@ kuwahara (GeglBuffer *src,
             gfloat variance = 0.0;
 
             compute_rectangle (src_buf,
-                               src->width,
-                               src->height,
+                               gegl_buffer_get_width (src),
+                               gegl_buffer_get_height (src),
                                u - radius -1,
                                v - radius -1,
                                1 + radius,
@@ -206,8 +171,8 @@ kuwahara (GeglBuffer *src,
               }
 
             compute_rectangle (src_buf,
-                               src->width,
-                               src->height,
+                               gegl_buffer_get_width (src),
+                               gegl_buffer_get_height (src),
                                u,
                                v - radius - 1,
                                1 + radius,
@@ -224,8 +189,8 @@ kuwahara (GeglBuffer *src,
               }
 
             compute_rectangle (src_buf,
-                               src->width,
-                               src->height,
+                               gegl_buffer_get_width (src),
+                               gegl_buffer_get_height (src),
                                u - radius - 1,
                                v,
                                1 + radius,
@@ -242,8 +207,8 @@ kuwahara (GeglBuffer *src,
               }
 
             compute_rectangle (src_buf,
-                               src->width,
-                               src->height,
+                               gegl_buffer_get_width (src),
+                               gegl_buffer_get_height (src),
                                u,
                                v,
                                1 + radius,
@@ -269,89 +234,12 @@ kuwahara (GeglBuffer *src,
   g_free (dst_buf);
 }
 
-
-#include <math.h>
-static GeglRectangle
-get_defined_region (GeglOperation *operation)
+static void tickle (GeglOperation *operation)
 {
-  GeglRectangle  result = {0,0,0,0};
-  GeglRectangle *in_rect = gegl_operation_source_get_defined_region (operation,
-                                                                     "input");
-
-  GeglChantOperation *blur = GEGL_CHANT_OPERATION (operation);
-  gint       radius = ceil(blur->radius);
-
-  if (!in_rect)
-    return result;
-
-  result = *in_rect;
-  if (result.width  != 0 &&
-      result.height  != 0)
-    {
-      result.x-=radius;
-      result.y-=radius;
-      result.width +=radius*2;
-      result.height +=radius*2;
-    }
-  
-  return result;
-}
-
-static GeglRectangle get_source_rect (GeglOperation *self,
-                                      gpointer       context_id)
-{
-  GeglChantOperation *blur   = GEGL_CHANT_OPERATION (self);
-  GeglRectangle            rect;
-  gint                radius;
- 
-  radius = ceil(blur->radius);
-
-  rect  = *gegl_operation_get_requested_region (self, context_id);
-  if (rect.width  != 0 &&
-      rect.height  != 0)
-    {
-      rect.x -= radius;
-      rect.y -= radius;
-      rect.width  += radius*2;
-      rect.height  += radius*2;
-    }
-
-  return rect;
-}
-
-static gboolean
-calc_source_regions (GeglOperation *self,
-                     gpointer       context_id)
-{
-  GeglRectangle need = get_source_rect (self, context_id);
-
-  gegl_operation_set_source_region (self, context_id, "input", &need);
-
-  return TRUE;
-}
-
-static GeglRectangle
-get_affected_region (GeglOperation *self,
-                     const gchar   *input_pad,
-                     GeglRectangle  region)
-{
-  GeglChantOperation *blur   = GEGL_CHANT_OPERATION (self);
-  gint                radius;
- 
-  radius = ceil(blur->radius);
-
-  region.x -= radius;
-  region.y -= radius;
-  region.width  += radius*2;
-  region.height  += radius*2;
-  return region;
-}
-
-static void class_init (GeglOperationClass *operation_class)
-{
-  operation_class->get_defined_region  = get_defined_region;
-  operation_class->get_affected_region = get_affected_region;
-  operation_class->calc_source_regions = calc_source_regions;
+  GeglOperationAreaFilter *area = GEGL_OPERATION_AREA_FILTER (operation);
+  GeglChantOperation      *filter = GEGL_CHANT_OPERATION (operation);
+  area->left = area->right = area->top = area->bottom =
+  ceil (filter->radius);
 }
 
 #endif
