@@ -3,7 +3,7 @@
  * GEGL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * GEGL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,9 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
  */
@@ -26,12 +24,13 @@ gegl_chant_int (maxiter, 0, 512, 128, "maximum number of iterations")
 
 #else
 
-#define GEGL_CHANT_SOURCE
 #define GEGL_CHANT_NAME           mandelbrot
-#define GEGL_CHANT_DESCRIPTION    "Mandelbrot renderer."
-
 #define GEGL_CHANT_SELF           "mandelbrot.c"
+#define GEGL_CHANT_DESCRIPTION    "Mandelbrot renderer."
 #define GEGL_CHANT_CATEGORIES     "render"
+
+#define GEGL_CHANT_SOURCE
+
 #include "gegl-chant.h"
 
 static gfloat mandel_calc(GeglChantOperation *self, gfloat x, gfloat y)
@@ -74,17 +73,12 @@ process (GeglOperation *operation,
   {
     GeglRectangle *result = gegl_operation_result_rect (operation, context_id);
     gfloat *buf;
+    gint pxsize;
 
-    output = g_object_new (GEGL_TYPE_BUFFER,
-                        "format",
-                        babl_format ("Y float"),
-                        "x",      result->x,
-                        "y",      result->y,
-                        "width",  result->width ,
-                        "height", result->height,
-                        NULL);
+    output = gegl_buffer_new (result, babl_format ("Y float"));
+    g_object_get (output, "px-size", &pxsize, NULL);
 
-    buf = g_malloc (gegl_buffer_pixels (output) * gegl_buffer_px_size (output));
+    buf = g_malloc (result->width * result->height * pxsize);
       {
         gfloat *dst=buf;
         gint y;
