@@ -60,7 +60,17 @@ class Section
         @name=""
     end
     def menu_entry
-        "#{@name}"
+        if @name == "GeglBuffer" or
+           @name == "GeglNode" or
+           @name == "GeglColor" or
+           @name == "GeglRectangle" or
+           @name == "GeglProcessor" or
+           @name == "XML" or
+           @name == "The GEGL API" then
+          return "<div style='padding-top:0.5em'>#{@name}</div>"
+        else
+          return "#{@name}"
+        end
     end
     def to_s
         ret = "#{@name}\n"
@@ -154,7 +164,6 @@ class Function
                    <div class='return_type'>#{self.return_type.sub(/const/,"<span class='const'>const</span>").gsub("*","<span class='pointer'>*</span>")}</div>
                    <div class='function_name'>#{@name}</div>
                    </div>"
-
         ret += "<div class='function_args'>"
 
         first=true
@@ -303,8 +312,11 @@ elements = []
 function = nil
 state = :none
 arg_no=0
+
+(ARGV.length-1).times {
+    |file_no|
 line_no=0
-IO.foreach(ARGV[0]) {
+IO.foreach(ARGV[file_no]) {
     |line|
     line_no = line_no+1
 
@@ -439,9 +451,10 @@ IO.foreach(ARGV[0]) {
         state=:none
     end
 }
+}
 
-if ARGV.length!=2
-    puts "usage: #{$0} <gegl.h> <output.html>"
+if ARGV.length<2
+    puts "usage: #{$0} <header1 [header2 ..]> <output.html>"
     exit
 end
 
@@ -455,7 +468,7 @@ File.open(ARGV[1], "w") {|file|
 =end
 
 
-File.open(ARGV[1], "w") {|file|
+File.open(ARGV[ARGV.length-1], "w") {|file|
 
 file.puts "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/tr/xhtml1/DTD/xhtml1-transitional.dtd\">
 <html>          
@@ -492,12 +505,10 @@ file.puts "<div class='toc'>
          <h3>Contents</h3>
       </div>
       <ul>
-        <li><a href='index.html'>GEGL</a></li>
-        <li>&nbsp;</li>
-        <li><a href='index.html#Documentation'>Documentation</a></li>
+        <li><a href='index.html' style='padding-top: 0.5em;'>GEGL</a></li>
+        <li><a href='index.html#Documentation' style='padding-top: 0.5em;'>Documentation</a></li>
         <li><a href='index.html#Glossary'>&nbsp;&nbsp;Glossary</a></li>
         <li><a href='operations.html'>&nbsp;&nbsp;Operations</a></li>
-        <li>&nbsp;</li>
 
 "
     elements.each { |element|
@@ -506,10 +517,39 @@ file.puts "<div class='toc'>
     file.puts "</ul></div>\n"
    
     file.puts "<div class='paper'><div class='content'>" 
-    elements.each {|element| file.puts "<div class='section' id='section_#{element.name.gsub(' ','_')}'><a name='#{element.name.gsub(' ','_')}'></a>#{element.to_html}</div>"
+    elements.each {|element| 
+        if !element.name.empty?
+        file.puts "<div class='section' id='section_#{element.name.gsub(' ','_')}'><a name='#{element.name.gsub(' ','_')}'></a>#{element.to_html}</div>"
+        end
     }
     file.puts "</div></div>
   </body>
 </html>"
 
 }
+
+File.open("gegl.devhelp", "w") {|file|
+    file.puts "<?xml version='1.0' encoding='utf-8' standalone='no'?>
+<!DOCTYPE book PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN' ''>
+<book xmlns='http://www.devhelp.net/book' title='GEGL Reference Manual' link='api.html' author='' name='gegl'>"
+    file.puts "<chapters>"
+    elements.each { |element|
+       if element.is_a? Section and !element.name.empty?
+         file.puts "<sub name='#{element.name}' link='api.html\##{element.name.gsub(' ','_')}'/>"
+       end
+    }
+    file.puts "</chapters>"
+    file.puts "<functions>"
+
+    elements.each { |element|
+       if element.is_a? Function and !element.name.empty?
+         file.puts "<function name='#{element.name}' link='api.html\##{element.name.gsub(' ','_')}'/>"
+       end
+       if element.is_a? Section and !element.name.empty? and element.name=~ /^Gegl/
+         file.puts "<function name='#{element.name}' link='api.html\##{element.name.gsub(' ','_')}'/>"
+       end
+    }
+
+    file.puts "</functions>"
+    file.puts "</book>"
+}
