@@ -2,9 +2,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#include "gegl-operation-filter.h"
-#include "gegl-operation-source.h"
-#include "gegl-operation-composer.h"
+#include "operation/gegl-operation-filter.h"
+#include "operation/gegl-operation-source.h"
+#include "operation/gegl-operation-composer.h"
 
 static GList *
 gegl_operations_build (GList *list, GType type)
@@ -211,7 +211,7 @@ main (gint    argc,
         }
     }
 
-  g_print (html_top);
+  g_print ("%s", html_top);
 
   g_print ("<div class='toc'><ul>\n");
   g_print ("<li><a href='index.html'>GEGL</a></li><li>&nbsp;</li>\n");
@@ -258,7 +258,7 @@ main (gint    argc,
   g_print ("</table>");
 
 
-  g_print (html_bottom);
+  g_print ("%s", html_bottom);
 
   g_list_free (operations);
   gegl_exit ();
